/*
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
*/

#include <linux/wireless.h>
#include <linux/netdevice.h> /* needed by iw_handler */
#include <linux/if_arp.h>
#include <net/iw_handler.h>

#include "islsm_ioctl.h"
#include "isl_sm.h"

#ifdef MADWIFI
#include "linux/version.h"
#include "net80211/ieee80211_linux.h"
#endif /* MADWIFI */

/* 
 * private ioctl to allow experimentation
 */

#ifndef MADWIFI
static int
islsm_ioctl_sendannouncedpkt(struct net_device *netdev, struct iw_request_info *info,	
			    void *w, char *extra)
{
#define MAX_PKT_LEN 200
//	struct p54u *sc = dev->priv;
	union iwreq_data *u = w;
	char *data;
	size_t pktsize;

	printk("Send announced packet called");

	if (u == 0)
	{
		printk("Null pointer given");
		return -ENOTSUPP;
	}

	pktsize = u->data.length;

	if (pktsize > MAX_PKT_LEN)
	{
		printk("Packet length too long !");
		return -ENOTSUPP;
	}

	
	data = u->data.pointer;

	/* note : this does not work with second-generation devices */
	islsm_outofband_msg(netdev, data, pktsize, 0);
	return 0;
}
#endif /* MADWIFI */

/* add ioctls to change the verbosity of the driver */


#define SUPPORTED_WIRELESS_EXT                  16

#ifdef CONFIG_NET_WIRELESS

#ifndef MADWIFI

/**** WARNING *****/
/* _ALL_ of these calls are bogus except for set_channel_freq */
/* I expect to complete statistics soon enough */

struct iw_statistics *
islsm_wireless_stats(struct net_device *ndev)
{
	struct islsm *priv = netdev_priv(ndev);

	/* completely bogus */
	return &priv->iwstatistics;
}


static int
prism54_commit(struct net_device *ndev, struct iw_request_info *info,
	       char *cwrq, char *extra)
{
	return 0;
}

static int
prism54_get_name(struct net_device *ndev, struct iw_request_info *info,
		 char *cwrq, char *extra)
{
	char *capabilities;

	capabilities = "IEEE 802.11b/g";	/* Default */
	
	strncpy(cwrq, capabilities, IFNAMSIZ);
	
	return 0;
}

static int
prism54_set_freq(struct net_device *netdev, struct iw_request_info *info,
                 struct iw_freq *fwrq, char *extra)
{
        struct islsm *islsm = netdev_priv(netdev);
	int c;
	static int channels[ISLSM_NR_CHANNELS] = { 0x96c, 0x971, 0x976, 0x97b, 0x980, 0x985, 0x98a, 0x98f, 0x994, 0x999, 0x99e, 0x9a3, 0x9a8, 0x9b4 };

	if (fwrq->m < 1000 && fwrq->m >= 1 && fwrq->m <= ISLSM_NR_CHANNELS)
		/* we have a valid channel number */
		c = fwrq->m;
	else
		/* we dont handle freq_to_channel yet */
		return -ENOTSUPP;
	
	islsm_freq_change(netdev, 
			 c, channels[c-1],
			 ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SCAN,
			 ISLSM_TX_CONTROL_CHANNEL_MAGIC2_SCAN);
	
	return islsm->err;
}

static int
prism54_get_freq(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_freq *fwrq, char *extra)
{
	fwrq->m = 272;
	fwrq->e = 0;

	return 0;
}

static int
prism54_set_mode(struct net_device *ndev, struct iw_request_info *info,
		 __u32 * uwrq, char *extra)
{
 	struct islsm *priv = netdev_priv(ndev);
	int err;
/* 	u32 mlmeautolevel = CARD_DEFAULT_MLME_MODE; */

	/* Let's see if the user passed a valid Linux Wireless mode */
	if (*uwrq > IW_MODE_MONITOR || *uwrq < IW_MODE_AUTO) {
		printk(KERN_DEBUG
		       "%s: %s() You passed a non-valid init_mode.\n",
		       ndev->name, __FUNCTION__);
		return -EINVAL;
	}

	/* only do something in case we're going 
	   into monitor mode */
	/* see islsm_init in usb_netdev for the corresponding place for
	   the madwifi callback */
	priv->iw_mode = *uwrq;

	switch (priv->iw_mode) {
	case IW_MODE_MONITOR :
		ndev->type = ARPHRD_IEEE80211_PRISM;
		err = islsm_set_filter(ndev,
				      ISLSM_TX_CONTROL_FILTER_MONITOR, ndev->broadcast,
				      0, 0, 0, 0, 0);
		break;
		
	default:
		ndev->type = ARPHRD_ETHER;
		err = islsm_set_filter(ndev,
				      ISLSM_TX_CONTROL_FILTER_NOTYPE, ndev->broadcast,
				      0, 0, 0, 0, 0);
	}
	
	if (err)
		islsm_dbg("error setting filter : %i", err);

	return 0;
}

/* Use mib cache */
static int
prism54_get_mode(struct net_device *ndev, struct iw_request_info *info,
		 __u32 * uwrq, char *extra)
{
/* 	islpci_private *priv = netdev_priv(ndev); */

/* 	BUG_ON((priv->iw_mode < IW_MODE_AUTO) || (priv->iw_mode > */
/* 						  IW_MODE_MONITOR)); */
	*uwrq = IW_MODE_INFRA;

	return 0;
}

/* we use DOT11_OID_EDTHRESHOLD. From what I guess the card will not try to
 * emit data if (sensitivity > rssi - noise) (in dBm).
 * prism54_set_sens does not seem to work.
 */

static int
prism54_set_sens(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
/* 	islpci_private *priv = netdev_priv(ndev); */
/* 	u32 sens; */

/* 	/\* by default  the card sets this to 20. *\/ */
/* 	sens = vwrq->disabled ? 20 : vwrq->value; */

/* 	return mgt_set_request(priv, DOT11_OID_EDTHRESHOLD, 0, &sens); */
	return 0;
}

static int
prism54_get_sens(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
/* 	islpci_private *priv = netdev_priv(ndev); */
/* 	union oid_res_t r; */
/* 	int rvalue; */

/* 	rvalue = mgt_get_request(priv, DOT11_OID_EDTHRESHOLD, 0, NULL, &r); */

	vwrq->value = 0;
	vwrq->disabled = (vwrq->value == 0);
	vwrq->fixed = 1;

	return 0;
}

static int
prism54_get_range(struct net_device *ndev, struct iw_request_info *info,
		  struct iw_point *dwrq, char *extra)
{
	struct iw_range *range = (struct iw_range *) extra;

	memset(range, 0, sizeof (struct iw_range));
	dwrq->length = sizeof (struct iw_range);

	/* set the wireless extension version number */
	range->we_version_source = SUPPORTED_WIRELESS_EXT;
	range->we_version_compiled = WIRELESS_EXT;

	/* Now the encoding capabilities */
	range->num_encoding_sizes = 3;
	/* 64(40) bits WEP */
	range->encoding_size[0] = 5;
	/* 128(104) bits WEP */
	range->encoding_size[1] = 13;
	/* 256 bits for WPA-PSK */
	range->encoding_size[2] = 32;
	/* 4 keys are allowed */
	range->max_encoding_tokens = 4;

	/* we don't know the quality range... */
	range->max_qual.level = 0;
	range->max_qual.noise = 0;
	range->max_qual.qual = 0;
	/* these value describe an average quality. Needs more tweaking... */
	range->avg_qual.level = -80;	/* -80 dBm */
	range->avg_qual.noise = 0;	/* don't know what to put here */
	range->avg_qual.qual = 0;

	range->sensitivity = 200;

	/* retry limit capabilities */
	range->retry_capa = IW_RETRY_LIMIT | IW_RETRY_LIFETIME;
	range->retry_flags = IW_RETRY_LIMIT;
	range->r_time_flags = IW_RETRY_LIFETIME;

	/* I don't know the range. Put stupid things here */
	range->min_retry = 1;
	range->max_retry = 65535;
	range->min_r_time = 1024;
	range->max_r_time = 65535 * 1024;

	/* txpower is supported in dBm's */
	range->txpower_capa = IW_TXPOW_DBM;

	return 0;
}

/* Set AP address*/

static int
prism54_set_wap(struct net_device *ndev, struct iw_request_info *info,
		struct sockaddr *awrq, char *extra)
{
/* 	islpci_private *priv = netdev_priv(ndev); */
/* 	char bssid[6]; */
/* 	int rvalue; */

/* 	if (awrq->sa_family != ARPHRD_ETHER) */
/* 		return -EINVAL; */

/* 	/\* prepare the structure for the set object *\/ */
/* 	memcpy(&bssid[0], awrq->sa_data, 6); */

/* 	/\* set the bssid -- does this make sense when in AP mode? *\/ */
/* 	rvalue = mgt_set_request(priv, DOT11_OID_BSSID, 0, &bssid); */

	return -EINPROGRESS;	/* Call commit handler */
}

/* get AP address*/

static int
prism54_get_wap(struct net_device *ndev, struct iw_request_info *info,
		struct sockaddr *awrq, char *extra)
{
	int rvalue;

	rvalue = 0;
//	awrq->sa_family = ARPHRD_ETHER;

	return rvalue;
}

static int
prism54_set_scan(struct net_device *dev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
	/* hehe the device does this automagicaly */
	return 0;
}


static const iw_handler prism54_handler[] = {
	(iw_handler) prism54_commit,	/* SIOCSIWCOMMIT */
	(iw_handler) prism54_get_name,	/* SIOCGIWNAME */
	(iw_handler) NULL,	/* SIOCSIWNWID */
	(iw_handler) NULL,	/* SIOCGIWNWID */
	(iw_handler) prism54_set_freq,	/* SIOCSIWFREQ */
	(iw_handler) prism54_get_freq,	/* SIOCGIWFREQ */
	(iw_handler) prism54_set_mode,	/* SIOCSIWMODE */
	(iw_handler) prism54_get_mode,	/* SIOCGIWMODE */
	(iw_handler) prism54_set_sens,	/* SIOCSIWSENS */
	(iw_handler) prism54_get_sens,	/* SIOCGIWSENS */
	(iw_handler) NULL,	/* SIOCSIWRANGE */
	(iw_handler) prism54_get_range,	/* SIOCGIWRANGE */
	(iw_handler) NULL,	/* SIOCSIWPRIV */
	(iw_handler) NULL,	/* SIOCGIWPRIV */
	(iw_handler) NULL,	/* SIOCSIWSTATS */
	(iw_handler) NULL,	/* SIOCGIWSTATS */
	(iw_handler) NULL,	//	prism54_set_spy,	/* SIOCSIWSPY */
	(iw_handler) NULL,	//	iw_handler_get_spy,	/* SIOCGIWSPY */
	(iw_handler) NULL,	//	iw_handler_set_thrspy,	/* SIOCSIWTHRSPY */
	(iw_handler) NULL,	//	iw_handler_get_thrspy,	/* SIOCGIWTHRSPY */
	(iw_handler) prism54_set_wap,	/* SIOCSIWAP */
	(iw_handler) prism54_get_wap,	/* SIOCGIWAP */
	(iw_handler) NULL,	/* -- hole -- */
	(iw_handler) NULL,	/* SIOCGIWAPLIST depreciated */
	(iw_handler) prism54_set_scan,	/* SIOCSIWSCAN */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_scan,	/* SIOCGIWSCAN */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_essid,	/* SIOCSIWESSID */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_essid,	/* SIOCGIWESSID */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_nick,	/* SIOCSIWNICKN */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_nick,	/* SIOCGIWNICKN */
	(iw_handler) NULL,	/* -- hole -- */
	(iw_handler) NULL,	/* -- hole -- */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_rate,	/* SIOCSIWRATE */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_rate,	/* SIOCGIWRATE */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_rts,	/* SIOCSIWRTS */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_rts,	/* SIOCGIWRTS */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_frag,	/* SIOCSIWFRAG */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_frag,	/* SIOCGIWFRAG */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_txpower,	/* SIOCSIWTXPOW */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_txpower,	/* SIOCGIWTXPOW */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_retry,	/* SIOCSIWRETRY */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_retry,	/* SIOCGIWRETRY */
	(iw_handler) NULL,	//	(iw_handler) prism54_set_encode,	/* SIOCSIWENCODE */
	(iw_handler) NULL,	//	(iw_handler) prism54_get_encode,	/* SIOCGIWENCODE */
	(iw_handler) NULL,	/* SIOCSIWPOWER */
	(iw_handler) NULL,	/* SIOCGIWPOWER */
};

static const struct iw_priv_args prism54_private_args[] = {
	[0] = { /* args for islsm_ioctl_sendannouncedpkt */
		.cmd = SIOCIWFIRSTPRIV,
		.set_args =
		((IW_PRIV_TYPE_MASK & IW_PRIV_TYPE_BYTE) | /* type of args */
		 (IW_PRIV_SIZE_MASK & MAX_PKT_LEN))         /* max number of args */
		& (~IW_PRIV_SIZE_FIXED),                   /* size is not fixed */
		.get_args = 0,	                           /* Type and number of args */
		.name = "set_announcedpkt",	           /* Name of the extension */
	},
};

static const iw_handler prism54_private_handler[] = {
		(iw_handler) islsm_ioctl_sendannouncedpkt,       /* SIOCWFIRSTPRIV+15 */
};

struct iw_handler_def islsm_iw_handler_def = {
	.num_standard = sizeof (prism54_handler) / sizeof (iw_handler),
	.num_private = sizeof (prism54_private_handler) / sizeof (iw_handler),
	.num_private_args =
	    sizeof (prism54_private_args) / sizeof (prism54_private_args[0]),
	.standard = (iw_handler *) prism54_handler,
	.private = (iw_handler *) prism54_private_handler,
	.private_args = (struct iw_priv_args *) prism54_private_args,
	.spy_offset = 0,//offsetof(islpci_private, spy_data),
};

#else /* MADWIFI */

/*
 * Return wireless extensions statistics.
 */
struct iw_statistics *
islsm_wireless_stats(struct net_device *dev)
{
	struct islsm *sc = dev->priv;
	struct ieee80211com *ic = &sc->sc_ic;
	struct iw_statistics *is = &sc->iwstatistics;

	ieee80211_iw_getstats(ic, is);
	/* add in statistics maintained by the driver */
/* 	is->discard.code += sc->sc_stats.ast_rx_badcrypt; */
/* 	is->discard.retries += sc->sc_stats.ast_tx_xretries; */
/* 	is->discard.misc += sc->sc_stats.ast_tx_encap */
/* 			 + sc->sc_stats.ast_tx_nonode */
/* 			 + sc->sc_stats.ast_tx_xretries */
/* 			 + sc->sc_stats.ast_tx_fifoerr */
/* 			 + sc->sc_stats.ast_tx_filtered */
/* 			 + sc->sc_stats.ast_tx_nobuf */
/* 			 + sc->sc_stats.ast_tx_nobufmgt; */
/* 			 ; */
/* 	is->miss.beacon = sc->sc_stats.ast_bmiss; */

	return &sc->iwstatistics;
}

#include <net/iw_handler.h>
/*
 * Bounce functions to get to the 802.11 code. These are
 * necessary for now because wireless extensions operations
 * are done on the underlying device and not the 802.11 instance.
 * This will change when there can be multiple 802.11 instances
 * associated with a device and we must have a net_device for
 * each so we can manipulate them individually.
 */
#define	ISLSM_CHAR_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 char *erq, char *extra)				\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_POINT_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 struct iw_point *erq, char *extra)			\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_PARAM_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 struct iw_param *erq, char *extra)			\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_SOCKADDR_BOUNCE(name)					\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 struct sockaddr *erq, char *extra)			\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_FREQ_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 struct iw_freq *erq, char *extra)			\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_U32_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 __u32 *erq, char *extra)				\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}
#define	ISLSM_VOID_BOUNCE(name)						\
static int								\
islsm_ioctl_##name(struct net_device *dev, struct iw_request_info *info,	\
	     	 void *erq, char *extra)				\
{									\
	struct islsm *sc = dev->priv;				\
	return ieee80211_ioctl_##name(&sc->sc_ic, info, erq, extra);	\
}

ISLSM_CHAR_BOUNCE(giwname)
ISLSM_POINT_BOUNCE(siwencode)
ISLSM_POINT_BOUNCE(giwencode)
ISLSM_PARAM_BOUNCE(siwrate)
ISLSM_PARAM_BOUNCE(giwrate)
ISLSM_PARAM_BOUNCE(siwsens)
ISLSM_PARAM_BOUNCE(giwsens)
ISLSM_PARAM_BOUNCE(siwrts)
ISLSM_PARAM_BOUNCE(giwrts)
ISLSM_PARAM_BOUNCE(siwfrag)
ISLSM_PARAM_BOUNCE(giwfrag)
ISLSM_SOCKADDR_BOUNCE(siwap)
ISLSM_SOCKADDR_BOUNCE(giwap)
ISLSM_POINT_BOUNCE(siwnickn)
ISLSM_POINT_BOUNCE(giwnickn)
ISLSM_FREQ_BOUNCE(siwfreq)
ISLSM_FREQ_BOUNCE(giwfreq)
ISLSM_POINT_BOUNCE(siwessid)
ISLSM_POINT_BOUNCE(giwessid)
ISLSM_POINT_BOUNCE(giwrange)
ISLSM_U32_BOUNCE(siwmode)
ISLSM_U32_BOUNCE(giwmode)
ISLSM_PARAM_BOUNCE(siwpower)
ISLSM_PARAM_BOUNCE(giwpower)
ISLSM_PARAM_BOUNCE(siwretry)
ISLSM_PARAM_BOUNCE(giwretry)
ISLSM_PARAM_BOUNCE(siwtxpow)
ISLSM_PARAM_BOUNCE(giwtxpow)
ISLSM_POINT_BOUNCE(iwaplist)
#ifdef SIOCGIWSCAN
ISLSM_POINT_BOUNCE(siwscan)
ISLSM_POINT_BOUNCE(giwscan)
#endif
ISLSM_VOID_BOUNCE(setparam)
ISLSM_VOID_BOUNCE(getparam)
ISLSM_VOID_BOUNCE(setkey)
ISLSM_VOID_BOUNCE(delkey)
ISLSM_VOID_BOUNCE(setmlme)
ISLSM_VOID_BOUNCE(setoptie)
ISLSM_VOID_BOUNCE(getoptie)
ISLSM_VOID_BOUNCE(addmac)
ISLSM_VOID_BOUNCE(delmac)
ISLSM_VOID_BOUNCE(chanlist)

/* Structures to export the Wireless Handlers */
static const iw_handler islsm_handlers[] = {
	(iw_handler) NULL,				/* SIOCSIWCOMMIT */
	(iw_handler) islsm_ioctl_giwname,			/* SIOCGIWNAME */
	(iw_handler) NULL,				/* SIOCSIWNWID */
	(iw_handler) NULL,				/* SIOCGIWNWID */
	(iw_handler) islsm_ioctl_siwfreq,			/* SIOCSIWFREQ */
	(iw_handler) islsm_ioctl_giwfreq,			/* SIOCGIWFREQ */
	(iw_handler) islsm_ioctl_siwmode,			/* SIOCSIWMODE */
	(iw_handler) islsm_ioctl_giwmode,			/* SIOCGIWMODE */
	(iw_handler) islsm_ioctl_siwsens,			/* SIOCSIWSENS */
	(iw_handler) islsm_ioctl_giwsens,			/* SIOCGIWSENS */
	(iw_handler) NULL /* not used */,		/* SIOCSIWRANGE */
	(iw_handler) islsm_ioctl_giwrange,		/* SIOCGIWRANGE */
	(iw_handler) NULL /* not used */,		/* SIOCSIWPRIV */
	(iw_handler) NULL /* kernel code */,		/* SIOCGIWPRIV */
	(iw_handler) NULL /* not used */,		/* SIOCSIWSTATS */
	(iw_handler) NULL /* kernel code */,		/* SIOCGIWSTATS */
	(iw_handler) NULL,				/* SIOCSIWSPY */
	(iw_handler) NULL,				/* SIOCGIWSPY */
	(iw_handler) NULL,				/* -- hole -- */
	(iw_handler) NULL,				/* -- hole -- */
	(iw_handler) islsm_ioctl_siwap,			/* SIOCSIWAP */
	(iw_handler) islsm_ioctl_giwap,			/* SIOCGIWAP */
	(iw_handler) NULL,				/* -- hole -- */
	(iw_handler) islsm_ioctl_iwaplist,		/* SIOCGIWAPLIST */
#ifdef SIOCGIWSCAN
	(iw_handler) islsm_ioctl_siwscan,			/* SIOCSIWSCAN */
	(iw_handler) islsm_ioctl_giwscan,			/* SIOCGIWSCAN */
#else
	(iw_handler) NULL,				/* SIOCSIWSCAN */
	(iw_handler) NULL,				/* SIOCGIWSCAN */
#endif /* SIOCGIWSCAN */
	(iw_handler) islsm_ioctl_siwessid,		/* SIOCSIWESSID */
	(iw_handler) islsm_ioctl_giwessid,		/* SIOCGIWESSID */
	(iw_handler) islsm_ioctl_siwnickn,		/* SIOCSIWNICKN */
	(iw_handler) islsm_ioctl_giwnickn,		/* SIOCGIWNICKN */
	(iw_handler) NULL,				/* -- hole -- */
	(iw_handler) NULL,				/* -- hole -- */
	(iw_handler) islsm_ioctl_siwrate,			/* SIOCSIWRATE */
	(iw_handler) islsm_ioctl_giwrate,			/* SIOCGIWRATE */
	(iw_handler) islsm_ioctl_siwrts,			/* SIOCSIWRTS */
	(iw_handler) islsm_ioctl_giwrts,			/* SIOCGIWRTS */
	(iw_handler) islsm_ioctl_siwfrag,			/* SIOCSIWFRAG */
	(iw_handler) islsm_ioctl_giwfrag,			/* SIOCGIWFRAG */
	(iw_handler) islsm_ioctl_siwtxpow,		/* SIOCSIWTXPOW */
	(iw_handler) islsm_ioctl_giwtxpow,		/* SIOCGIWTXPOW */
	(iw_handler) islsm_ioctl_siwretry,		/* SIOCSIWRETRY */
	(iw_handler) islsm_ioctl_giwretry,		/* SIOCGIWRETRY */
	(iw_handler) islsm_ioctl_siwencode,		/* SIOCSIWENCODE */
	(iw_handler) islsm_ioctl_giwencode,		/* SIOCGIWENCODE */
	(iw_handler) islsm_ioctl_siwpower,		/* SIOCSIWPOWER */
	(iw_handler) islsm_ioctl_giwpower,		/* SIOCGIWPOWER */
};
static const iw_handler islsm_priv_handlers[] = {
	(iw_handler) islsm_ioctl_setparam,		/* SIOCWFIRSTPRIV+0 */
	(iw_handler) islsm_ioctl_getparam,		/* SIOCWFIRSTPRIV+1 */
	(iw_handler) islsm_ioctl_setkey,			/* SIOCWFIRSTPRIV+2 */
	(iw_handler) NULL,				/* SIOCWFIRSTPRIV+3 */
	(iw_handler) islsm_ioctl_delkey,			/* SIOCWFIRSTPRIV+4 */
	(iw_handler) NULL,				/* SIOCWFIRSTPRIV+5 */
	(iw_handler) islsm_ioctl_setmlme,                /* SIOCWFIRSTPRIV+6 */
	(iw_handler) NULL,				/* SIOCWFIRSTPRIV+7 */
	(iw_handler) islsm_ioctl_setoptie,		/* SIOCWFIRSTPRIV+8 */
	(iw_handler) islsm_ioctl_getoptie,		/* SIOCWFIRSTPRIV+9 */
	(iw_handler) islsm_ioctl_addmac,			/* SIOCWFIRSTPRIV+10 */
	(iw_handler) NULL,				/* SIOCWFIRSTPRIV+11 */
	(iw_handler) islsm_ioctl_delmac,			/* SIOCWFIRSTPRIV+12 */
	(iw_handler) NULL,				/* SIOCWFIRSTPRIV+13 */
	(iw_handler) islsm_ioctl_chanlist,		/* SIOCWFIRSTPRIV+14 */
};

struct iw_handler_def islsm_iw_handler_def = {
#define	N(a)	(sizeof (a) / sizeof (a[0]))
	.standard		= (iw_handler *) islsm_handlers,
	.num_standard		= N(islsm_handlers),
	.private		= (iw_handler *) islsm_priv_handlers,
	.num_private		= N(islsm_priv_handlers),
#undef N
};

#endif /* MADWIFI */

#endif /* CONFIG_NET_WIRELESS */
