/*
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
*/

#ifndef _HAVE_PROTOCOL_H
#define _HAVE_PROTOCOL_H

#include <linux/types.h>
#include <linux/if_ether.h>
#include <linux/netdevice.h>

/* These should be read from the mgmt frame
   I don't really know where they are located...
*/
#define ISLSM_NR_CHANNELS 14
#define ISLSM_NR_RATES 12 /* from what i can read in the packets */

/* this seems to be present 
   in all kinds of frames 
   received on data and mgmt pipes */
struct islsm_rx {
	u16     length;
	u8      padding[14];
	char    data[];
} __attribute__((packed));

#define SIZE_RX sizeof(struct islsm_rx)

/* the tx'd packets start with a destination on 32 bits, we'll call that
   their "queue" hereafter */
struct islsm_tx 
{
	u32 queue;
/* until i sort out what this means... */
#define ISLSM_TX_QUEUE_CONFIG     0x00020200
#define ISLSM_TX_QUEUE_ZERO       0x0002006c
#define ISLSM_TX_QUEUE_ZERO_VER2  0x00020070

//#define ISLSM_TX_CONTROL_FILTER_MAGIC6_VER1 0x02480c
//#define ISLSM_TX_CONTROL_FILTER_MAGIC6_VER2 0x024214
//#define ISLSM_TX_CONTROL_FILTER_MAGIC6_PCI  0x024790


/*
The variable part is at the same location in v2 devices
0x00020670
0x00020770
*/
	u16 length;
#define ISLSM_MGMT_FOLLOWUP    0x2
	u16 follower;
	u8  padding[8];
}  __attribute__ ((packed));

#define SIZE_NET2280_H sizeof(struct islsm_tx)

/*
 * structure accounting for the different queues a prio queue would
 * probably be better. Must be able to link between queue and packet, so
 * that a tx timeout can reclaim a tx'd packet asynchronously (does the
 * 802.11 stack allow this ?)
 */
#define ISLSM_TX_QUEUE_MASK       0xF
#define ISLSM_TX_QUEUE_TX(x) (((x & ISLSM_TX_QUEUE_MASK) << 8) | islsm->datatx_reg)
#define ISLSM_TX_QUEUE_ID(x) (((x) >> 8) & ISLSM_TX_QUEUE_MASK)
/* this is observed maximum, i really don't know */
#define ISLSM_NR_TX_QUEUES 4

struct tx_queue {
	u8 id;       /* one of 06, 07, 08, 09 : are there more ? */
	void *ptr;   /* pointer to some data structure needed (urb, skb...) */

        /* the usb id of the packet sent. This allows
	 * tracking of what tx_queue is freed by a 
	 * struct islsm_control_queueemptied packet 
	 */
	/* stored in "raw" device format */
	u32 usb_id;
};

/* helper functions for tx queue management */
u8    islsm_txqueue_alloc(struct net_device *netdev, void* ptr);
void *islsm_txqueue_free(struct net_device *netdev, u32 usb_id);

/* pimfor from the main driver. */
/* Guess what : same size, 12 bytes */
/*
typedef struct {
	u8 version;
	u8 operation;
	u32 oid;
	u8 device_id;
	u8 flags;
	u32 length;
} __attribute__ ((packed))
pimfor_header_t;
*/

/* the length should be the length of data *after* the header we're in */
struct islsm_id {
	u16    magic1;
/* TX magic numbers */
#define ISLSM_TX_MGMT_ID   0x4000
#define ISLSM_TX_BEACON_ID 0x4002
/* sometimes data packets begin like this */
#define ISLSM_TX_DATA_ID   0x4010

/* Actually i think all bits in the first byte are meaningless and are
 * copied indiscriminately in the response packet. But looking at data
 * transactions, i'm not so confident... */
#define ISLSM_TX_CONTROL_NORESPONSE_ID  0x8001
#define ISLSM_ID_EEPROM_READBACK        0x8000
#define ISLSM_ID_FAULT                  0x4d00

#define ISLSM_MGMT_FRAME              (1<<15)
#define ISLSM_DATA_FRAME              (1<<14)
/*
 * Set this bit if you don't want any response on the mgmt pipe. Usually
 * this is OK for settings only echoing their result (rx/tx filter
 * packets, frequency change packets 
 */
/* get = 0 / set = 1 
   must try get with zeroed buffer (i didn't try this !) !!!  */
#define ISLSM_ID_NO_MGMT_RESPONSE     (1 << 0)
/* this flag is an unknown quantity for me.
   seems usefull only for sending data  */
#define ISLSM_ID_UNKNOWN              (1 << 4)

/* The following magic numbers are coming along with data packets.
   We need to understand better what they mean so we get rid of this
   list */
#define ISLSM_ID_DATA_PKT (0)
// maybe means trap response too ?
#define ISLSM_ID_FCS_OK   (1 << 0)
#define ISLSM_ID_MASK     (0xff00)

/* provide an example capture */
/* seen in non-monitor mode : 0x4c,0x54,0x5c,0x52 */
/* seen in real monitor mode : 0xb0,0xa0,0x84,0x80,0x70,
   0x74,0x60,0x64,0x4c,0x48,0x44,0x40,0x04,0x00 */
#define ISLSM_DATA_PKT_ACK  (1<<7)
	u16    length;
} __attribute__ ((packed));

/* this is from the main driver. Guess what. 20 bytes. */
struct rfmon_header {
	u16 unk0;		/* = 0x0000 */
	u16 length;		/* = 0x1400 */
	u32 clock;		/* 1MHz clock */
	u8 flags;
	u8 unk1;
	u8 rate;
	u8 unk2;
	u16 freq;
	u16 unk3;
	u8 rssi;
	u8 padding[3];
} __attribute__ ((packed));

/* packets wrapping a received 802.11 packet */
struct islsm_rx_packet_header {
	/* This first field to be put above */
	struct islsm_id id;
	u16     freq;   /* frequency, in MHz, le */
	u16     unknown1; /* not always zero. What's the information there ? */
	u8      signal_strength; /* , in ?, le. Not sure about the meaning of the first byte, really */
	u8      padding2;
	u16     unknown2;
	u32     timestamp; /* timestamp, in microsec, le */
	u32     padding;
} __attribute__ ((packed));

#define SIZE_RX_HEADER sizeof(struct islsm_rx_packet_header)

/* the above islsm_tx and islsm_rx headers often wrap this kind of packets */
struct islsm_tx_control
{
	struct islsm_id id;
	/* req_d precision : The device doesn't like them to be
	 * mangled. Be sure that they are _unique in the device_ ! it's
	 * used internally too, so sending two packets with the same
	 * req_id will confuse the device. Beware ! */
	u32     req_id;
	/* actually this is 2 16-bit fields */
	u16     type;
#define ISLSM_TX_CONTROL_TYPE_FILTER_SET        0x0
#define ISLSM_TX_CONTROL_TYPE_DATANOACK	       0x0
#define ISLSM_TX_CONTROL_TYPE_CHANNEL_CHANGE    0x1
#define ISLSM_TX_CONTROL_TYPE_DATAACK	       0x1
#define ISLSM_TX_CONTROL_TYPE_FREQDONE          0x2
#define ISLSM_TX_CONTROL_TYPE_FREEQUEUE         0x7
#define ISLSM_TX_CONTROL_TYPE_TXDONE            0x8
#define ISLSM_TX_CONTROL_TYPE_STAT_READBACK     0xa
#define ISLSM_TX_CONTROL_TYPE_EEPROM_READBACK   0xc
#define ISLSM_TX_CONTROL_TYPE_LED               0xd

#define ISLSM_TX_CONTROL_TYPE_TXVER2            0x8000
	u16     flags;
#define ISLSM_TX_CONTROL_FLAGS_ACK              0x0707
#define ISLSM_TX_CONTROL_FLAGS_NOACK	       0x0101

	char    data[];
} __attribute__ ((packed));

#define SIZE_TX_CONTROL sizeof(struct islsm_tx_control)

/* subtypes neede for mgmt readback frame */

struct islsm_freq_struct3
{
	u16 freq;
	u16 data[4];
} __attribute__ ((packed));

struct islsm_mgmt_rx_header 
{
	u32 magic1;
	u16 padding1;
	u16 preamble_length; /* this is null for linksys devices, but is not null for SMC dongles, for instance */
	u32 padding2;
	char data[];
} __attribute__ ((packed));

#define MGMT_MAX_FRAME_SIZE 0x100
struct islsm_mgmt_option
{
	/* length of magic + data, in 16 bit words */
	u16 length;
	u16 magic;
#define MGMT_NAME_OPTION     0x0001
#define MGMT_SERIAL_OPTION   0x0003

#define MGMT_MAC_OPTION      0x0101

#define MGMT_HWUNKW_OPTION   0x1001
#define MGMT_HWTYPE_OPTION   0x1002

#define MGMT_FREQ_3_OPTION   0x1903 // array, with array length
#define MGMT_FREQ_4_OPTION   0x1904 // array, with array length, padded
#define MGMT_FREQ_5_OPTION   0x1905 
#define MGMT_FREQ_6_OPTION   0x1906 // may be an array, without array length
#define MGMT_FINALCRC_OPTION 0x0000
#define MGMT_END_OPTION      0xffff
	char data[];
} __attribute__ ((packed));

struct islsm_hw_option
{
	u8  unk1;
#define HW_UNK1 0x03
	u8  isl;
#define HW_3887 0x87
#define HW_3880 0x86
#define HW_3886 0x80
	u16 unk2;
#define HW_UNK2 0x5501
} __attribute__ ((packed));

struct islsm_fw_option
{
	/* length of data, in 32 bit words, le */
	u32 magic;
	u32 length;
#define LMAC_OPT_1 0x1
#define LMAC_OPT_2 0x2
#define LMAC_OPT_3 0x4
#define LMAC_OPT_4 0x101
#define LMAC_OPT_FINAL 0xff
#define LMAC_OPT_FLAG (1<<31)
	char data[];
} __attribute__ ((packed));

/**** MGMT_FREQ3 ****/

struct islsm_freq_struct1
{
	u16 freq; /* in MHz */
	u8 data[6];
} __attribute__ ((packed));

struct islsm_mgmt_freq3_option
{
	/* number of elements */
	u8 flags;
	u8 nelem;
	struct islsm_freq_struct1 data[];
} __attribute__ ((packed));

/**** MGMT_FREQ4 ****/

/* uncomplete information */
struct islsm_freq_option4_s
{
	u16 head;
	u8 tail;
} __attribute__ ((packed));

struct islsm_freq_option4_f
{
	u16 head;
	u8 subtails[4];
	u8 tail;
} __attribute__ ((packed));

/* complete information, in command packet */
#define ISLSM_CHANNEL_DEFS 8
struct islsm_freq_substruct2
{
	struct islsm_freq_option4_f command;
	u8 padding; /* separator; zero */
} __attribute__ ((packed));

struct islsm_freq_option4
{
	u16 freq;
	char data[];
} __attribute__ ((packed));

struct islsm_mgmt_freq4_option
{
	/* seen as 1 on Nicolas's device. maybe size of padding, in
	 * option of array ? -- of extended or not ? */
#define FREQ4_TYPE_SHORT    0x0
	/* seen on Nicolas' device */
#define FREQ4_TYPE_EXTENDED 0x1
	u8 flags;
	u8 nfreqs;
	/* this nsettings is 7 for nicolas' device, 8 for others. Is
	 * this the number of antenna, for instance ? */
	u8 nsettings;
	u8 type;
// Type here depends on above type and flags
	char data[];
} __attribute__ ((packed));

/* 
 * Frequency setting packet
 */
struct islsm_tx_control_channel
{
	u16 magic1;
	u16 magic2;
/* Setting when passive-scanning */
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SCAN          0x2
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC2_SCAN          0x78
// OR with default scan values when reaching channel 14
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SCAN_WRAP     0x8

/* Setting when active-scanning.
 * Such a frequency change packet triggers
 * the emission of the packet in slot ISLSM_FRAME_PROBE
 */
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC1_EMIT_TXSLOT    0x6
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC2_EMIT_TXSLOT    0x28

/* TX mode. When in this mode, subsequent TX packets with mode
 * ISLSM_FRAME_DATA, ISLSM_FRAME_MGMT, ISLSM_FRAME_BEACON are directly
 * emitted. A correct tx/rx filter is needed for BEACON type.
 */
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC1_TX            0x1
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC2_TX              0

/* This mode is used in ad-hoc mode, when you want to emit a packet
 * (null data in dabe's log) while still scanning. Is this possible ?
 * See for instance Dabe's log.
 */
#define ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SET2          0x3
	u8  padding1[20];
	/* This is the channel definition, coming directly from the
	 * MGMT_FREQ6_OPTION */
	struct islsm_freq_struct3 freq;

	/* I guess this is the number of predefined settings for the
	 * radio. I only have one problem, i don't know what each of
	 * these configuration means, nor how to select which one...  */
	/* number of available settings and initial values are found
	 * during the mgmt readback in the MGMT_FREQ4_OPTION */
	u8 nsettings;

	/* Following 5 8-bits fields are unknown. They seem to come from
	 * the MGMT_FREQ_3_OPTION */
	u8 unknown1; /* field unknown, the only one -- well no, goes
		      * with the following ones, i'd say */
	u8  l1data[4]; /* filled with values from l1 */

	/* The settings proper, zero-separated */
	struct islsm_freq_substruct2 str[ISLSM_CHANNEL_DEFS];
	u32 padding; /* filled with 0 in old firmware, repeat of l1data
		      * in newer devices */
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_CHANNEL sizeof(struct islsm_tx_control_channel)
#define FULL_TX_CONTROL_CHANNEL SIZE_TX_CONTROL + SIZE_TX_CONTROL_CHANNEL

struct islsm_tx_control_channel_ver2
{
	struct islsm_tx_control_channel basic_packet;
	u32 padding;
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_CHANNEL_VER2 sizeof(struct islsm_tx_control_channel_ver2)
#define FULL_TX_CONTROL_CHANNEL_VER2 SIZE_TX_CONTROL + SIZE_TX_CONTROL_CHANNEL_VER2

/* 
 * packet which sets the rx/tx filter
 */
struct islsm_tx_control_filter
{
	u16 filter_type;
#define ISLSM_TX_CONTROL_FILTER_NOTYPE    0
#define ISLSM_TX_CONTROL_FILTER_STA       (1<<0)
#define ISLSM_TX_CONTROL_FILTER_ADHOC     (1<<1)
#define ISLSM_TX_CONTROL_FILTER_MONITOR   (1<<4)
	u8  destination[ETH_ALEN];
	u8  source[ETH_ALEN];
	u16 magic2;
// FIXME / TODO : understand what those fucking fields mean. If not
// zeroed, version2 devices are goind completely askew.
// Do cache-coloring in order to understand if their value is
// meaningfull in version1 devices.
// needs active testing in monitor more, i *think* that we have 
// a filter bitfield here. But not sure.
#define ISLSM_TX_CONTROL_FILTER_MAGIC2_FORM1 0 //xde02
#define ISLSM_TX_CONTROL_FILTER_MAGIC2_FORM2 0 //xbd02
#define ISLSM_TX_CONTROL_FILTER_MAGIC2_FORM3 0 //xbd00
#define ISLSM_TX_CONTROL_FILTER_MAGIC2_STA     0 //xbd02
#define ISLSM_TX_CONTROL_FILTER_MAGIC2_MONITOR 0x0000
	u32 magic3;
#define ISLSM_TX_CONTROL_FILTER_MAGIC3_FORM1   0
#define ISLSM_TX_CONTROL_FILTER_MAGIC3_STA     0x15f
#define ISLSM_TX_CONTROL_FILTER_MAGIC3_MONITOR 0x0
	u32 magic4;
/* I think this is rate control... */
#define ISLSM_TX_CONTROL_FILTER_MAGIC4_FORM1 0x01040608
#define ISLSM_TX_CONTROL_FILTER_MAGIC4_FORM2 0x010303
#define ISLSM_TX_CONTROL_FILTER_MAGIC4_STA     0x0
#define ISLSM_TX_CONTROL_FILTER_MAGIC4_MONITOR 0x0
	u32 magic5;
	u32 magic6;
/* I love blogspeak */
/* OMFG GET RID OF THIS !!!??!!! */
#define ISLSM_TX_CONTROL_FILTER_MAGIC6_VER1 0x02480c
#define ISLSM_TX_CONTROL_FILTER_MAGIC6_VER2 0x024214
#define ISLSM_TX_CONTROL_FILTER_MAGIC6_PCI  0x024790
	u32 magic7;
#define ISLSM_TX_CONTROL_FILTER_MAGIC7_VER1 0x030610
/* seen on pci for david's device */
#define ISLSM_TX_CONTROL_FILTER_MAGIC7_VER2 0x030620
#define ISLSM_TX_CONTROL_FILTER_MAGIC7_PCI  0x010620
	u16 magic8;
#define ISLSM_TX_CONTROL_FILTER_MAGIC8_STA   0x1f4
	u16 magic9;
/* this is a time i think. Find out what it controls */
#define ISLSM_TX_CONTROL_FILTER_MAGIC9_FORM1 0xf642
#define ISLSM_TX_CONTROL_FILTER_MAGIC9_PCI   0xf642
	/* pfui ! */
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_FILTER sizeof(struct islsm_tx_control_filter)
#define FULL_TX_CONTROL_FILTER SIZE_TX_CONTROL + SIZE_TX_CONTROL_FILTER

/*
 * packet which controls the leds 
 */
struct islsm_tx_control_led {
/* The workings of the mode is not really clear. Basically,
   a value of 1 will set the green led to blink on packet rx,
   all other values will go out of this mode */
	u16 mode;
#define LED_MODE_SET       2
#define LED_MODE_RXBLINK   1

/* the leds will be under control of led_setting_temporary 
   for temporary_setting_duration time, in msec, then they
   are set according to led_setting_permanent.
   This is for both leds.
   For instance, if you want the green led to blink while the 
   red led is still on, be sure to set it to LED_GREEN | LED_RED
   otherwise the red led will be off during the blink */
	u16 led_setting_temporary;
	u16 led_setting_permanent;
#define LED_RED   1
#define LED_GREEN 2

	/* duration of the temporary settings, in ms */
	u16 temporary_setting_duration;
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_LED sizeof(struct islsm_tx_control_led)
#define FULL_TX_CONTROL_LED SIZE_TX_CONTROL + SIZE_TX_CONTROL_LED

/*
 * packet which is sent to free a 
 * queue where a frame has been alloc'd
 */

struct islsm_tx_control_freequeue {
	u32 queue;
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_FREEQUEUE sizeof(struct islsm_tx_control_freequeue)
#define FULL_TX_CONTROL_FREEQUEUE SIZE_TX_CONTROL + SIZE_TX_CONTROL_FREEQUEUE

/*
 * Data sending packet
 */ 

struct islsm_wepkey {
	u8      key_present;
#define ISLSM_WEPKEY_PRESENT 0x1
	u8      key_length;
	u8      key[16];
} __attribute__ ((packed));

struct islsm_tx_control_allocdata {
/* I'd bet this is the rate */
	u32     magic1;
#define ISLSM_TX_FRAME_MAGIC1_UNSPEC	0x0
#define ISLSM_TX_FRAME_MAGIC1_DATA       0x04060708
	u32     magic2;
	u16     padding;
	struct  islsm_wepkey wepkey;
	u32     frame_type;
#define ISLSM_FRAME_DATA	                0x4
#define ISLSM_FRAME_MGMT            	0x2
#define ISLSM_FRAME_PROBE                0x1
#define ISLSM_FRAME_BEACON               0x0
	u32     padding2;
	u32     magic4;
	u32     magic5;
/* 02 7f 23 with 00 40 magic seems okay for id purposes */
/* 02 is for some kind of mode ? */
#define ISLSM_TX_FRAME_MAGIC4_MGMT	0x7f020000 /* 0x7f000000, other
						    * form with 00 40 as
						    * magic, and 0x23
						    * below (dabe's device) */
#define ISLSM_TX_FRAME_MAGIC5_MGMT	0x23 /* 0x33, other form with 10
					      * 40 as magic, and 7f02 above */
#define ISLSM_TX_FRAME_MAGIC4_PROBE	0x0
#define ISLSM_TX_FRAME_MAGIC5_PROBE	0x0
	/* 4 is the max padding seen */
	u8      frame_align[];
} __attribute__ ((packed));

#define SIZE_TX_CONTROL_ALLOCDATA sizeof(struct islsm_tx_control_allocdata)
#define FULL_TX_CONTROL_ALLOCDATA SIZE_TX_CONTROL + SIZE_TX_CONTROL_ALLOCDATA
#define TX_MAX_PADDING 4

#define ISLSM_SIZE_STATS 0x24

/*
 * eeprom readback packet
 */
struct islsm_control_eeprom {
	u16	offset;
	u16	len;
	char    data[];
} __attribute__ ((packed));

#define SIZE_CONTROL_EEPROM sizeof(struct islsm_control_eeprom)
#define FULL_CONTROL_EEPROM SIZE_TX_CONTROL + SIZE_TX_CONTROL_ALLOCDATA

/*
 * packet reporting successfull frequency setting
 */
struct islsm_control_freqchange {
	u16      padding;
	u16      frequency;
} __attribute__ ((packed));

/*
 * packet sent when a data packet is freed
 */ 
struct islsm_control_queueemptied {
	u16      status;
/* i'll have to make sure of this */
#define PKT_STATUS_OK        0x100
#define PKT_STATUS_NOTACKED  0x1
	u16      ack_strength;
	/* the device takes care of the sequence control field
	   in the packet. Here is what it filled it with */
	u16      sequence;
	/* this is, i think another field filled in by the device.
	   duration / ID field, maybe ? Seems to be field 0x22-0x23
	   repeated */
	u16      unknown;
} __attribute__ ((packed));

/*
 * packet received, statistics
 */

struct islsm_control_stats {
	u32 rx_pkt;
	u32 stat1; /* counter */
	u32 stat2; /* counter */
	u32 stat3; /* counter */
	u32 stat4;
	u32 stat5;
	u32 timestamp; /* in microseconds, see rx packet */
	u32 stat6;
	u32 noise; /* noise ? -- i'm not sure, anyone checks this ?*/
} __attribute__ ((packed));

/* now the helper functions, for sending packets */
int islsm_outofband_msg(struct net_device *netdev,
			void *buf, unsigned int size, unsigned lmac_addr);

int islsm_stats_readback(struct net_device *netdev);

void islsm_make_control_led(struct net_device *netdev, char *buf,
			   u16 mode, u16 perm_setting, u16 temp_setting, u16 duration);

void islsm_make_freequeue(struct net_device *netdev, char* buf,
			 u8 queue_id);

int islsm_set_filter(struct net_device *netdev, 
		    u16 filter_type, u8 *bssid,
		    u16 magic2, u32 magic3, u32 magic4, 
		    u16 magic8, u16 magic9);

int islsm_freq_change(struct net_device *netdev, u16 channel, u16 freq, 
		     u16 magic1, u16 magic2);

int islsm_data_tx(struct net_device *netdev, unsigned short padding,
		 u32 id_magic1, u16 req_type, u16 flags,
		 u32 magic4, u32 magic5,
		 u32 frame_type, 
		 struct sk_buff *skb);

int islsm_mgmt_readback(struct net_device *netdev);

int islsm_empty_queues(struct net_device *netdev);

/* now the helper functions, for receiving packets */
void islsm_data_input(struct net_device *netdev, struct sk_buff * skb);
void islsm_bootup_input(struct net_device *netdev, struct sk_buff * skb);
void islsm_parse_mgmt_frame(struct net_device *netdev);

/* lmac management */
void  islsm_txqueue_init(struct net_device *netdev);
int   islsm_parse_firmware(struct net_device *netdev, void* data, unsigned int len);

#endif /* _HAVE_PROTOCOL_H */
