/*
  Copyright 2004 Feyd
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
*/

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/smp_lock.h>
#include <linux/completion.h>
#include <linux/usb.h>
#include <linux/pci.h>
#include <linux/firmware.h>
#include <asm/uaccess.h>

// get rid of this here, it's only needed for the alloc_etherdev ?
#include <linux/netdevice.h>
#include <linux/etherdevice.h>

#include "isl_38xx.h"
#include "islusb_dev.h"
#include "isl_sm.h"

#ifdef CONFIG_ISLUSB_DEBUG
int islusb_debug = 2;
#else
int islusb_debug = 0;
#endif

int load_fw;
static const char driver_name[] = "islsm_usb";

static const struct usb_device_id p54u_table[] = {
	{USB_DEVICE(0x5041, 0x2234)},	/* Linksys WUSB54G */
	{USB_DEVICE(0x5041, 0x2235)},	/* Linksys WUSB54G Portable */
	{USB_DEVICE(0x1915, 0x2234)},	/* Linksys WUSB54G OEM */
	{USB_DEVICE(0x1915, 0x2235)},	/* Linksys WUSB54G Portable OEM */
	{USB_DEVICE(0x0506, 0x0a11)},	/* 3COM 3CRWE254G72 */
	{USB_DEVICE(0x2001, 0x3701)},	/* DLink DWL-G120 Spinnaker */
	{USB_DEVICE(0x2001, 0x3703)},	/* DLink DWL-G122 */
	{USB_DEVICE(0x0cde, 0x0006)},	/* Medion 40900, Roper Europe */
	{USB_DEVICE(0x0846, 0x4200)},	/* Netgear WG121 */
	{USB_DEVICE(0x0846, 0x4210)},	/* Netgear WG121 the second ? */
	{USB_DEVICE(0x0846, 0x4220)},	/* Netgear WG111 */
	{USB_DEVICE(0x0707, 0xee06)},	/* SMC 2862W-G */
	{USB_DEVICE(0x124a, 0x4023)},   /* Shuttle PN15, Airvast WM168g, IOGear GWU513 */
	{USB_DEVICE(0x083a, 0x4501)},   /* Accton 802.11g WN4501 USB */
	{USB_DEVICE(0x083a, 0x4502)},   /* Siemens Gigaset USB Adapter */

/* Version 2 devices, experimental support */
	{USB_DEVICE(0x0846, 0x4240)},   /* Netgear WG111, recent version */
	{USB_DEVICE(0x0cde, 0x0008)},   /* Sagem XG703A */

/* Devices from the Tivo driver ; don't know which are second
 * generation and which first generation, but they should be supported */

	{USB_DEVICE(0x09AA, 0x1000)},   /* Spinnaker Proto board */
	{USB_DEVICE(0x413C, 0x8102)},   /* Spinnaker DUT */
	{USB_DEVICE(0x0846, 0x4240)},   /* Netgear WG111 */
	{USB_DEVICE(0x050D, 0x7050)},   /* Belkin F5D7050 ver 1000 */
	{USB_DEVICE(0x0915, 0x2000)},   /* Cohiba Proto board */
	{USB_DEVICE(0x0915, 0x2002)},   /* Cohiba Proto board */
	{USB_DEVICE(0x0572, 0x2000)},   /* Cohiba Proto board */
	{USB_DEVICE(0x0572, 0x2002)},   /* Cohiba Proto board */
	{USB_DEVICE(0x413C, 0x8104)},   /* Cohiba Proto board */
	{USB_DEVICE(0x0D8E, 0x3762)},   /* DLink DWL-G120 Cohiba */
	{}
};

static const struct p54u_pipe_desc p54u_pipes[] = {
	{ P54U_PIPE_DATA,		USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_MGMT,		USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_BRG,		USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_DEV,		USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_DATA | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_MGMT | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_3    | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_4    | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_BRG  | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_DEV  | USB_DIR_IN,	USB_ENDPOINT_XFER_BULK },
	{ P54U_PIPE_INT  | USB_DIR_IN,	USB_ENDPOINT_XFER_INT  },
};


MODULE_DESCRIPTION("Prism54 USB Driver");
MODULE_AUTHOR("Feyd <feyd@seznam.cz>, Jean-Baptiste Note <jean-baptiste.note@m4x.org>");
MODULE_LICENSE("GPL");
MODULE_DEVICE_TABLE(usb, p54u_table);
MODULE_PARM(islusb_debug, "i");
MODULE_PARM(load_fw, "i");
MODULE_PARM_DESC(islusb_debug, "Debug enabled or not for usb");
MODULE_PARM_DESC(load_fw, "Load firmware on probe (1) or open (0)");

static
void islusb_mdelay(int ms)
{
	int t = (ms * HZ + 500) / 1000;
	
	if(!t && ms)
		t = 1;
	
	set_current_state(TASK_UNINTERRUPTIBLE);
	while(t) {
		//printk("t: %i\n", t);
		t = schedule_timeout(t);
	}
	
	return;
}

static int p54u_reset_pipe(struct usb_device *usbdev, int addr)
{
  int err = 0;
  
  islusb_dbg("resetting pipe %02x", addr);
  switch(addr) {
  case P54U_PIPE_DATA:
  case P54U_PIPE_MGMT:
  case P54U_PIPE_BRG :
    err = usb_clear_halt(usbdev, usb_sndbulkpipe(usbdev,addr) );
    break;
  case P54U_PIPE_BRG | USB_DIR_IN :
  case P54U_PIPE_DATA | USB_DIR_IN:
  case P54U_PIPE_MGMT | USB_DIR_IN:
    err = usb_clear_halt(usbdev, usb_rcvbulkpipe(usbdev,addr) );
    break;
  case P54U_PIPE_INT | USB_DIR_IN:
    err = usb_clear_halt(usbdev, usb_rcvintpipe(usbdev,addr) );
    break;
  default:
    break;
  }
  return err;
}

int p54u_reset_pipes(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	struct usb_device *usbdev = p54u->usbdev;

	(void) p54u_reset_pipe(usbdev, P54U_PIPE_DEV | USB_DIR_IN );
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_DEV);
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_BRG | USB_DIR_IN );
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_BRG);

	(void) p54u_reset_pipe(usbdev, P54U_PIPE_INT | USB_DIR_IN );

	(void) p54u_reset_pipe(usbdev, P54U_PIPE_DATA | USB_DIR_IN );
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_DATA);
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_MGMT | USB_DIR_IN );
	(void) p54u_reset_pipe(usbdev, P54U_PIPE_MGMT);

	return 0;
}

int p54u_alloc_buffers(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	struct usb_interface *interface = p54u->interface;
	struct usb_host_interface *iface_desc = &interface->altsetting[0];
	struct usb_endpoint_descriptor *desc;
	int err = 0, i;
	
	islusb_info("%s: Setup USB structures.\n", netdev->name);
	
	for(i = 0; i < iface_desc->desc.bNumEndpoints; ++i) {
		desc = &iface_desc->endpoint[i].desc;
		switch(desc->bEndpointAddress) {
		case P54U_PIPE_DATA:
			err = p54u_queue_init(netdev, desc, &p54u->data_tx);
			break;
		case P54U_PIPE_MGMT:
			err = p54u_queue_init(netdev, desc, &p54u->mgmt_tx);
			break;
		case P54U_PIPE_DATA | USB_DIR_IN:
			err = p54u_queue_init(netdev, desc, &p54u->data_rx);
			break;
		case P54U_PIPE_MGMT | USB_DIR_IN:
			err = p54u_queue_init(netdev, desc, &p54u->mgmt_rx);
			break;
		case P54U_PIPE_INT | USB_DIR_IN:
			err = p54u_queue_init(netdev, desc , &p54u->int_rx);
			break;
		default:
			islusb_info("unallocated %02x endpoint",desc->bEndpointAddress);	
			break;
		}
		if(err)
			goto do_err;
	}
	islusb_info("%s: Setup USB structures successful.\n", netdev->name);
	return 0;
	
    do_err:
    	islusb_info("%s: Setup USB structures failed: %i\n", netdev->name, err);
//	p54u_free_buffers(netdev);
	
	return err;
}

static int p54u_reset_dev(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	u32 reg, revision;
	int err;
	
	islusb_info("%s: Reset device.\n", netdev->name);
	
	/* Bridge */
	/* Reset the usb<->pci bridge */
	islusb_dbg("%s: Reset bridge", netdev->name);
	reg = p54u_brg_readl(netdev, NET2280_GPIOCTL);
	if (reg & P54U_BRG_POWER_UP)
		islusb_dbg("%s: bridge already up, expect problems", netdev->name);
	//reg = 0x3e;
	//reg |= 0x3e;
	p54u_brg_writel(netdev, NET2280_GPIOCTL, (reg | P54U_BRG_POWER_DOWN) & ~P54U_BRG_POWER_UP);
	if(p54u->err)
		goto do_err;
	islusb_mdelay(100);
	p54u_brg_writel(netdev, NET2280_GPIOCTL, (reg | P54U_BRG_POWER_UP) & ~P54U_BRG_POWER_DOWN);
	if(p54u->err)
		goto do_err;
	islusb_dbg("%s: Reset bridge done\n", netdev->name);
	islusb_mdelay(100);
	
	/* See 11.5.1 on net2280 doc */
	islusb_dbg("%s: Magic 1\n", netdev->name);
	p54u_brg_writel(netdev, NET2280_DEVINIT, NET2280_CLK_30Mhz | NET2280_PCI_ENABLE | NET2280_PCI_SOFT_RESET);
	if(p54u->err)
		goto do_err;
	islusb_dbg("%s: Magic 1 done\n", netdev->name);
	islusb_mdelay(20);
	
	/* Enable mmio and busmaster on the bridge */
	islusb_dbg("%s: Setup bridge pci resources\n", netdev->name);
	p54u_pcicfg_brg_writew(netdev, PCI_COMMAND, (PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));
	
	/* Set base address 0 */
	p54u_pcicfg_brg_writel(netdev, PCI_BASE_ADDRESS_0, NET2280_BASE);

	/* Set PCI_STATUS_REC_MASTER_ABORT (why?) */
	reg = p54u_pcicfg_brg_readw(netdev, PCI_STATUS);
	p54u_pcicfg_brg_writew(netdev, PCI_STATUS, (reg | PCI_STATUS_REC_MASTER_ABORT));
	
	/* Read revision? */
	revision = p54u_brg_readl(netdev, NET2280_RELNUM);
	islusb_dbg("%s: Setup bridge pci resources done\n", netdev->name);
	
	/* Magic */
	islusb_dbg("%s: Magic 2\n", netdev->name);
	p54u_brg_writel(netdev, NET2280_EPA_RSP, NET2280_CLEAR_NAK_OUT_PACKETS_MODE);
	p54u_brg_writel(netdev, NET2280_EPC_RSP, NET2280_CLEAR_NAK_OUT_PACKETS_MODE);
	islusb_dbg("%s: Magic 2 done\n", netdev->name);
	
	/* Set base address 2 for bridge : where the endpoint fifos are */
	islusb_dbg("%s: Setup bridge base addr 2\n", netdev->name);
	p54u_pcicfg_brg_writel(netdev, PCI_BASE_ADDRESS_2, NET2280_BASE2);
	islusb_dbg("%s: Setup bridge base addr 2 done\n", netdev->name);

	/* Device */
	/* Enable mmio and busmaster on the device */
	islusb_dbg("%s: Setup device pci resources\n", netdev->name);
	p54u_pcicfg_dev_writew(netdev, PCI_COMMAND | 0x10000, (PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));
//	p54u_pcicfg_dev_writew(netdev, PCI_COMMAND, (PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));
	
	/* Set TRDY_TIMEOUT and RETRY_TIMEOUT to 0 */
	p54u_pcicfg_dev_writew(netdev, P54U_TRDY_TIMEOUT | 0x10000, 0);
//	p54u_pcicfg_dev_writew(netdev, P54U_TRDY_TIMEOUT, 0);
	
	/* Set base address 0 */
	p54u_pcicfg_dev_writel(netdev, PCI_BASE_ADDRESS_0 | 0x10000, P54U_DEV_BASE);
//	p54u_pcicfg_dev_writel(netdev, PCI_BASE_ADDRESS_0, P54U_DEV_BASE);
	islusb_dbg("%s: Setup device pci resources done\n", netdev->name);
	
	/* See 7.6.5.5 and chapter 9 of net2280 doc */
	islusb_dbg("%s: Magic 3\n", netdev->name);
	p54u_brg_writel(netdev, NET2280_USBIRQENB1, 0);
//	p54u_brg_writel(netdev, NET2280_USBIRQENB1, NET2280_PCI_INTA_INTERRUPT_ENABLE | NET2280_USB_INTERRUPT_ENABLE);
	p54u_brg_writel(netdev, NET2280_IRQSTAT1, NET2280_PCI_INTA_INTERRUPT);
	islusb_dbg("%s: Magic 3 done\n", netdev->name);
	
	/* Assert wakeup */
	islusb_dbg("%s: Assert device\n", netdev->name);
	p54u_dev_writel(netdev, ISL38XX_DEV_INT_REG,  ISL38XX_DEV_INT_WAKEUP);
	if(p54u->err)
		goto do_err;
	islusb_mdelay(20);
	
	/* (see old driver for INT_ABORT definition) */
	p54u_dev_writel(netdev, ISL38XX_DEV_INT_REG,  ISL38XX_DEV_INT_ABORT);
	if(p54u->err)
		goto do_err;
	islusb_dbg("%s: Assert device done\n", netdev->name);
	islusb_mdelay(20);
	
	/* Reset the device */
	islusb_dbg("%s: Reset device\n", netdev->name);

	/* clear the RAMBoot, the CLKRUN and the Reset bit */
	reg = p54u_dev_readl(netdev, ISL38XX_CTRL_STAT_REG);
//	reg &= ~(ISL38XX_CTRL_STAT_RESET | ISL38XX_CTRL_STAT_RAMBOOT);
	reg &= ~(ISL38XX_CTRL_STAT_RESET | ISL38XX_CTRL_STAT_RAMBOOT | ISL38XX_CTRL_STAT_CLKRUN);
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg);
	islusb_mdelay(20); /* WRITEIO_DELAY */

	/* set the Reset bit without reading the register */
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg | ISL38XX_CTRL_STAT_RESET);
	islusb_mdelay(20); /* WRITEIO_DELAY */
	/* clear the Reset bit */
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg);

	if(p54u->err)
		goto do_err;
	islusb_dbg("%s: Reset device done\n", netdev->name);
	/* wait for the device to reboot */
	islusb_mdelay(100);
	islusb_dbg("%s: Disable irq\n", netdev->name);
	p54u_dev_writel(netdev, ISL38XX_INT_EN_REG, 0);
//	p54u_dev_writel(netdev, ISL38XX_INT_EN_REG, 0x00004004);
	islusb_dbg("%s: Disable irq done\n", netdev->name);
	
//	islusb_mdelay(50);
	
	/* Ack the irqs */
	islusb_dbg("%s: Ack irq\n", netdev->name);
	reg = p54u_dev_readl(netdev, ISL38XX_INT_IDENT_REG);
	p54u_dev_writel(netdev, ISL38XX_INT_ACK_REG, reg);
	if(p54u->err)
		goto do_err;
	islusb_dbg("%s: Ack done\n", netdev->name);

	/* Magic */
	islusb_dbg("%s: Magic 4\n", netdev->name);
	p54u_brg_writel(netdev, NET2280_EPA_STAT, NET2280_FIFO_FLUSH);
	p54u_brg_writel(netdev, NET2280_EPB_STAT, NET2280_FIFO_FLUSH);
	p54u_brg_writel(netdev, NET2280_EPC_STAT, NET2280_FIFO_FLUSH);
	p54u_brg_writel(netdev, NET2280_EPD_STAT, NET2280_FIFO_FLUSH);
	/* why this again ? */
	p54u_brg_writel(netdev, NET2280_EPA_STAT, NET2280_FIFO_FLUSH);
	islusb_dbg("%s: Magic 4 done\n", netdev->name);

	islusb_info("%s: Reset device done.\n", netdev->name);
	
	return 0;
	
    do_err:
    	islusb_err("%s: Reset error: %i\n", netdev->name, p54u->err);
    	err = p54u->err;
	p54u->err = 0;
	return err;
}

static int p54u_reset_dev2(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	static char* init_string = "~~~~";
	/* really simple : start the urbs on data pipe,
	   send reinit sequence, and you're all set */
	islusb_dbg("%s: device reset start\n", netdev->name);
	p54u->err = rx_queue_refill(netdev, &p54u->data_rx);
	p54u->err = p54u_bulk_msg(netdev, P54U_PIPE_DATA, init_string, strlen(init_string));
	// give him device to respond. Should wait on sem ?
	// NB : that's what joerg's driver does ; wait 100 ms.
	// Bad coders !
	islusb_mdelay(100);
	islusb_dbg("%s: device reset complete\n", netdev->name);
	return p54u->err;
}

static int p54u_load_firmware(struct net_device *netdev, const struct firmware *fw_entry)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	u32 reg;
	void *data, *fw_buf;
	size_t len;
	int err, i, j;

	islusb_info("%s: Load firmware.\n", netdev->name);

	fw_buf = kmalloc(P54U_FW_BLOCK,GFP_KERNEL);
	if (!fw_buf)
		return -ENOMEM;

	/* Firmware */
	data = fw_entry->data;
	i = fw_entry->size;
	j = 0;

	while(i > 0) {
		len = i > P54U_FW_BLOCK ? P54U_FW_BLOCK : i;

		memcpy(fw_buf,data,len);
		p54u->err = p54u_bulk_msg(netdev, P54U_PIPE_DATA, fw_buf, len);

		if(p54u->err) {
			islusb_dbg("%s: Error writing firmware block: writen %i of %i.\n", netdev->name, j, fw_entry->size);
			goto do_release;
		}

		data += P54U_FW_BLOCK;
		i -= P54U_FW_BLOCK;
		
		/* Magic */
		p54u_dev_writel(netdev, ISL38XX_DIR_MEM_BASE_REG, 0xc0000f00);
		p54u_dev_writel(netdev, 0x1020, 0);
		p54u_dev_writel(netdev, 0x1020, 1);
		p54u_dev_writel(netdev, 0x1024, len);
		p54u_dev_writel(netdev, 0x1028, j | 0x00020000);
		p54u_dev_writel(netdev, 0x0060, 0x20000000);
		p54u_dev_writel(netdev, 0x0064, len/4);
		p54u_dev_writel(netdev, 0x0068, 4);
		reg = p54u_dev_readl(netdev, 0x102c);
		if(i > 0)
			p54u_brg_writel(netdev, NET2280_EPA_STAT, NET2280_FIFO_FLUSH);
		if(p54u->err) {
			islusb_dbg("%s: Error updating registers: writen %i of %i.\n", netdev->name, j, fw_entry->size);
			goto do_release;
		}
		j += P54U_FW_BLOCK;

	}
	
	reg = p54u_dev_readl(netdev, ISL38XX_CTRL_STAT_REG);
//	reg |= ISL38XX_CTRL_STAT_RAMBOOT | ISL38XX_CTRL_STAT_CLKRUN;
	reg |= ISL38XX_CTRL_STAT_RAMBOOT;
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg);
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg | ISL38XX_CTRL_STAT_RESET);
	p54u_dev_writel(netdev, ISL38XX_CTRL_STAT_REG, reg);
	
	if(p54u->err) {
		islusb_dbg("%s: Error latching reset: %i\n", netdev->name, p54u->err);
	}

	islusb_info("%s: Load firmware done.\n", netdev->name);
	kfree(fw_buf);
	return 0;
	
    do_release:
    	islusb_err("%s: Load error: %i\n", netdev->name, p54u->err);
    	err = p54u->err;
	p54u->err = 0;
	kfree(fw_buf);
	return err;
}

/* tries to write write bytes from data to buf */
/* the data needs some formatting : 
 * escape 0x7e to 0x7d 0x5e
 * escape 0x7d to 0x7d 0x5d
 */
static 
unsigned firmware_escape_write_buffer(char *buf, char *data, unsigned int write)
{
	unsigned int i = 0,j = 0;
	char val;
	while ( j < write )
	{
		switch ( val = data[i++] )
		{
		case '~' :
			buf[j++] = '}';
			buf[j++] = '^';
			break;
		case '}' :
			buf[j++] = '}';
			buf[j++] = ']';
			break;
		default:
			buf[j++] = val;
		}
	}
	/* problem if we escape the last character ! -- we should
	 * rewrite the whole thing to avoid this case... or be able to
	 * pass a carry... Someone fixes this ! */
	BUG_ON(j != write);
	return i;
}

static int p54u_load_firmware2(struct net_device *netdev,
			       const struct firmware *fw_entry) {
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	void *data, *fw_buf;

	static char* start_string = "~~~~<\r";
	static char* go_string = "y   g\r";
	unsigned int length, read, written, remains;
	
	fw_buf = kmalloc(P54U_FW_BLOCK,GFP_KERNEL);
	if (!fw_buf)
		return -ENOMEM;

	/* this is awfully simple */
	data = fw_entry->data;
	remains = fw_entry->size;

	/* copy the command */
	strcpy(fw_buf,start_string);
	length = strlen(start_string);

	islusb_info("%s: Load firmware.\n", netdev->name);

	while ( remains > 0 && (p54u->err == 0))
	{
		written = remains > (P54U_FW_BLOCK-length) ? (P54U_FW_BLOCK-length) : remains;
		read = firmware_escape_write_buffer(fw_buf + length, data, written);

		/* directly transfert the data */
		p54u->err = p54u_bulk_msg(netdev, P54U_PIPE_DATA, fw_buf, length + written);
		
		if(p54u->err) {
			islusb_dbg("%s: Error writing firmware block: remains %i of %i.\n", netdev->name, remains, fw_entry->size);
		}

		data += read;
		remains -= read;
		length = 0;
	}

	islusb_info("%s: Load firmware done, sending go.\n", netdev->name);

	/* send GO */
	/* maybe 79 is y, e18f is a length, 95 an opcode */
#define JUMP_ADDR 0x8fe19579
	strcpy(fw_buf,go_string);
	*(u32 *)(fw_buf) = cpu_to_le32(JUMP_ADDR);
	
	p54u->err = p54u_bulk_msg(netdev, P54U_PIPE_DATA, fw_buf, strlen(go_string));
	islusb_info("%s: Sending go done.\n", netdev->name);

	/* wait for OK and g */
	p54u->err = islsm_wait_timeout(islsm,HZ);
	
	kfree(fw_buf);
	
	return p54u->err;
}

int p54u_setup_dev(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;

	/* now use the new RX queue mechanism */
	/* Those functions can fail : no way to bail out ? */
	rx_queue_refill(netdev, &p54u->int_rx);
	rx_queue_refill(netdev, &p54u->data_rx);
	rx_queue_refill(netdev, &p54u->mgmt_rx);
	
	//p54u_dev_writel(netdev, ISL38XX_INT_EN_REG, 0x80004004);
	p54u_dev_writel(netdev, ISL38XX_INT_EN_REG, 0x00004004);
	p54u_brg_writel(netdev, NET2280_IRQSTAT1, NET2280_PCI_INTA_INTERRUPT);
	p54u_brg_writel(netdev, NET2280_USBIRQENB1, NET2280_PCI_INTA_INTERRUPT_ENABLE | NET2280_USB_INTERRUPT_ENABLE);
	p54u_dev_writel(netdev, ISL38XX_DEV_INT_REG, ISL38XX_DEV_INT_RESET);
	
	p54u->err = islsm_wait_timeout(islsm,HZ);
	return p54u->err;
}

/* kicks the hardware state machine on */
int p54u_boot(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	const struct firmware	*fw_entry;
	int err = 0;
	int try = 0;

	/* do nothing if we're already up */
	if (p54u->running == 1)
		return 0;

	islusb_info("%s: Boot the device.\n", netdev->name);

	/* Allocations. Here only software can fault */
	p54u->state = P54U_BOOT;

	err = request_firmware(&fw_entry, 
			       p54u->device_version == P54U_DEVICE_VER1 ? P54U_IMAGE_FILE : P54U_IMAGE_FILE_X2, 
			       netdev->class_dev.dev);

	if(err) {
		islusb_err("%s: Request firmware failed: %i\n", netdev->name, err);
		return err;
	}
		
	islusb_dbg("%s: Request firmware done: len = %i.\n", netdev->name, fw_entry->size);

	// then we should check if we're okay
	// This begs for X2 header generation !
#define X2_LEN 0x10
	(void) islsm_parse_firmware(netdev, (p54u->device_version == P54U_DEVICE_VER1) ? fw_entry->data : fw_entry->data + X2_LEN, fw_entry->size);

	if(!err)
		err = p54u_alloc_buffers(netdev);

	/* usb operations */
 try_again:
	try++;
	if(!err) {
		if (p54u->device_version == P54U_DEVICE_VER1)
			err = p54u_reset_dev(netdev);
		if (p54u->device_version == P54U_DEVICE_VER2)
			err = p54u_reset_dev2(netdev);
	}
	if(!err) {
		if (p54u->device_version == P54U_DEVICE_VER1)
			err = p54u_load_firmware(netdev,fw_entry);
		if (p54u->device_version == P54U_DEVICE_VER2)
			err = p54u_load_firmware2(netdev,fw_entry);
	}
	// no need for this step in ver2
	if(!err && p54u->device_version == P54U_DEVICE_VER1)
		err = p54u_setup_dev(netdev);
 	if(!err) 
	{
		/* change state, now that the prism chip has booted */
		p54u->state = P54U_RUN;
		err = islsm_mgmt_readback(netdev);
	}
	if(!err)
		islsm_parse_mgmt_frame(netdev);

	if (err && try <= 3)
	{
		err = 0;
		p54u->err = 0;
		islusb_mdelay(100);
		goto try_again;
	}

	if(err) {
		islusb_err("%s: Boot the device failed: %i\n", netdev->name, err);
		goto exit_release_firmware;
	}

	/* those flags have no fucking meaning */
	p54u->running = 1;

	islusb_info("%s: Boot the device done", netdev->name);

	exit_release_firmware :
		release_firmware(fw_entry);
	return err;
}

static int p54u_validate_device(struct usb_interface *interface, const struct usb_device_id *id)
{
	/* We'll check that the device has the required endpoints in at
	 * least one of its alternate configurations. returns -1 in case there's
	 * not, otherwise, the index of the first alternate
	 * configuration which matches what we need */
		
	struct usb_endpoint_descriptor *epdesc;
	struct usb_host_interface *curint = interface->altsetting;
	unsigned int i,j;
	islusb_info("%i alternate settings for device",interface->num_altsetting);
	for (i=0 ; i < interface->num_altsetting; i++)
	{
		unsigned int recognized_pipes = 0;
		
		for(j=0; j < curint->desc.bNumEndpoints; j++)
		{
			epdesc = &curint->endpoint[j].desc;
			switch(epdesc->bEndpointAddress) {
			case P54U_PIPE_DATA:
			case P54U_PIPE_MGMT:
			case P54U_PIPE_BRG:
			case P54U_PIPE_DEV:
			case P54U_PIPE_DATA | USB_DIR_IN:
			case P54U_PIPE_MGMT | USB_DIR_IN:
			case P54U_PIPE_BRG | USB_DIR_IN:
			case P54U_PIPE_DEV | USB_DIR_IN:
			case P54U_PIPE_INT | USB_DIR_IN:
				recognized_pipes++;
				islusb_info("recognized endpoint %02x",epdesc->bEndpointAddress);
				break;
			default:
				islusb_info("unrecognized endpoint %02x",epdesc->bEndpointAddress);
			}
		}
		if (recognized_pipes >= P54U_PIPE_NUMBER)
		{
			islusb_info("Alternate setting %i ok for driver",i);
			return i;
		}
		curint++;
	}
	return -1;
}

/* manages allocation of the netdev and 802.11 resources */
static int p54u_probe(struct usb_interface *interface, const struct usb_device_id *id)
{
	struct usb_device *usbdev = interface_to_usbdev(interface);
	struct net_device *netdev;
	struct p54u *p54u;
	struct islsm *islsm;
	int conf_index;

	usb_get_dev(usbdev);

	islusb_dbg("Prism54 USB Device Probe (Device number:%d): 0x%4.4x:0x%4.4x:0x%4.4x", usbdev->devnum, (int) usbdev->descriptor.idVendor, (int) usbdev->descriptor.idProduct, (int) usbdev->descriptor.bcdDevice);
	islusb_dbg("Device at %p", usbdev);
	islusb_dbg("Descriptor length: %x type: %x", (int) usbdev->descriptor.bLength, (int) usbdev->descriptor.bDescriptorType);

	conf_index = p54u_validate_device(interface, id);

        // do this allocation elsewhere ?
	netdev = alloc_etherdev(sizeof(struct islsm));
	if (!netdev) {
		islusb_err("Failed to allocate netdevice.");
		return -ENOMEM;
	}

	islsm = netdev_priv(netdev);
	memset(islsm, 0, sizeof(*islsm));

	/* allocates p54u */
	p54u = (struct p54u *) kmalloc(sizeof(*p54u),GFP_KERNEL);
	if (!p54u) {
		islusb_err("Failed to allocate p54u device");
		return -ENOMEM;
	}
	memset(p54u, 0, sizeof(*p54u));
	islsm->hardware = p54u;

	// i seem to recall that there is a way to get to this with
	// offsetof ?
	p54u->interface = interface;
	p54u->usbdev = usbdev;
	p54u->netdev = netdev;

	SET_MODULE_OWNER(netdev);	

	// FIXME
	usb_set_intfdata(interface, islsm);

	/* version-dependent initialization */
	if (conf_index == 0)
	{
		islusb_info("Suitable configuration found for version 1 device");
		p54u->device_version = 	P54U_DEVICE_VER1;

		islsm->config_reg = ISLSM_TX_QUEUE_CONFIG;
		islsm->datatx_reg = ISLSM_TX_QUEUE_ZERO;
		islsm->isl_boot = p54u_boot;
		islsm->isl_tx = tx_submit_ver1;
		islsm->device_tx_header_space = SIZE_NET2280_H;
		islsm->device_version = ISLSM_DEVICE_USB_VER1;
	}
	else
	{
		islusb_info("No suitable configuration found, trying experimental device v2 support");
		p54u->device_version = 	P54U_DEVICE_VER2;

		/* some if not all of these should be filled
		   automatically when parsing the firmware */
		islsm->config_reg = ISLSM_TX_QUEUE_CONFIG;
		islsm->datatx_reg = ISLSM_TX_QUEUE_ZERO_VER2;
		islsm->isl_boot = p54u_boot;
		islsm->isl_tx = tx_submit_ver2;
		islsm->device_tx_header_space = 0;
		islsm->device_version = ISLSM_DEVICE_USB_VER2;
	}

        // usb_put_dev(usbdev);
	// I don't know what this means, if this is needed at all...
	SET_NETDEV_DEV(netdev, &interface->dev);

	if(islsm_setup_net(netdev)) {
		islusb_err("Failed to setup netdevice");
		goto do_cleanup;
	}	

	if(load_fw)
		/* Dont error if it fails, give it chance on open. */
		islsm->isl_boot(netdev);

	return 0;
	
    do_cleanup:
	usb_set_intfdata(interface, NULL);
	free_netdev(netdev);
	usb_put_dev(usbdev);
	return -EIO;
}

static void p54u_disconnect(struct usb_interface *interface)
{
	struct usb_device *usbdev = interface_to_usbdev(interface);
	struct islsm *islsm = usb_get_intfdata(interface);
	struct p54u *p54u = (struct p54u *) islsm->hardware;
	struct net_device *netdev = p54u->netdev;

	islusb_info("Prism54 USB device %s disconnecting", netdev->name);

	islsm_unsetup_net(netdev);
	islusb_dbg("Stopped software state machines");

	// Once the madwifi is detached, there is no bh problem any more
	// so this is okay. Previously there were heavy problems with
	// RUN->INIT transition that left bh disabled when running this.

	if (p54u->running == 1)
	{
		p54u->state = P54U_SHUTDOWN;
		p54u_queue_destroy(&p54u->data_rx);
		p54u_queue_destroy(&p54u->mgmt_rx);
		p54u_queue_destroy(&p54u->data_tx);
		p54u_queue_destroy(&p54u->mgmt_tx);
		if (p54u->device_version == P54U_DEVICE_VER1)
			p54u_queue_destroy(&p54u->int_rx);
		p54u->running = 0;
		islusb_info("All queues destroyed");
	}

	usb_set_intfdata(interface, NULL);
	islusb_dbg("Cleared interface data");

	free_netdev(netdev);
	islusb_dbg("Freed netdevice");

	// could be done earlier
	usb_put_dev(usbdev);

	islusb_info("Disconnect complete");
}

static struct usb_driver p54u_driver = {
	.owner = THIS_MODULE,
	.name = driver_name,
	.probe = p54u_probe,
	.disconnect = p54u_disconnect,
	.id_table = p54u_table,
};

static int __init p54u_init(void)
{
	int result;

	/* register this driver with the USB subsystem */
	result = usb_register(&p54u_driver);
	if (result) {
		islusb_err("usb_register failed. Error number %d", result);
		return result;
	}

	return 0;
}

static void __exit p54u_exit(void)
{
	/* deregister this driver with the USB subsystem */
	usb_deregister(&p54u_driver);
}

module_init(p54u_init);
module_exit(p54u_exit);

