/*
	rt2x00 SourceForge Project - <http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	Licensed under the GNU GPL
	Original code supplied under license from RaLink Inc, 2003
 */

/*
	Module Name: rt2x00_compatibility.h

	Abstract: Defines to make the module backward compatible with older kernels.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		<releasedate>	Initial Release.
 */

#ifndef __RT2X00_COMPATIBILITY_H
#define __RT2X00_COMPATIBILITY_H

#include <linux/version.h>

#ifndef MODULE_VERSION
#define MODULE_VERSION(__version)
#endif /* MODULE_VERSION */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#define try_module_get(__x)	MOD_INC_USE_COUNT
#define module_put(__x)		MOD_DEC_USE_COUNT
#define usb_get_dev(__dev)	usb_inc_dev_use(__dev)
#define usb_put_dev(__dev)	usb_dec_dev_use(__dev)
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
static inline struct ethhdr *eth_hdr(const struct sk_buff *skb)
{
	return (struct ethhdr *)skb->mac.raw;
}
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9) */

#ifndef __iomem
#define __iomem
#endif /* __iomem */

#ifndef pci_name
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define pci_name(__pPci_Dev)	(__pPci_Dev)->dev.bus_id
#else /* (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) */
#define pci_name(__pPci_Dev)	(__pPci_Dev)->slot_name
#endif /*(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) */
#endif /* pci_name */

#ifndef HAVE_NETDEV_PRIV
#define netdev_priv(__netdev)		( (__netdev)->priv )
#endif /* HAVE_NETDEV_PRIV */

#ifndef HAVE_FREE_NETDEV
#define free_netdev(__netdev)		( kfree(__netdev) )
#endif /* HAVE_FREE_NETDEV */

#ifndef IRQ_NONE
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif /* IRQ_NONE */

#ifndef dma_alloc_coherent
#define dma_alloc_coherent(__dev, __size, __dma, __flag) \
	( pci_alloc_consistent((__dev), (__size), (__dma)) )
#endif /* dma_alloc_coherent */

#ifndef dma_free_coherent
#define dma_free_coherent(__dev, __size, __cpu, __dma) \
	( pci_free_consistent((__dev), (__size), (__cpu), (__dma)) )
#endif /* dma_free_coherent */

#ifndef SET_NETDEV_DEV
#define SET_NETDEV_DEV(__net, __dev)
#endif /* SET_NETDEV_DEV */

#ifndef PCI_DEVICE
#define PCI_DEVICE(vend,dev) \
	.vendor = (vend), .device = (dev), \
	.subvendor = PCI_ANY_ID, .subdevice = PCI_ANY_ID
#endif /* PCI_DEVICE */

#ifndef DMA_64BIT_MASK
#define DMA_64BIT_MASK  0xffffffffffffffffULL
#endif /* DMA_64BIT_MASK */

#ifndef DMA_32BIT_MASK
#define DMA_32BIT_MASK  0x00000000ffffffffULL
#endif /* DMA_32BIT_MASK */

#endif /* __RT2X00_COMPATIBILITY_H */
