/*
	rt2x00 SourceForge Project - <http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	Licensed under the GNU GPL
	Original code supplied under license from RaLink Inc, 2003
 */

/*
	Module Name: rt2x00_mgmt.h

	Abstract: Header structures and details for management packets.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		<releasedate>	Initial Release.
 */
  
#ifndef __RT2X00_MGMT_H
#define __RT2X00_MGMT_H

#define IEEE802_11_MGMT_HLEN		24

struct ieee802_11_mgmt_hdr {
	u16	frame_ctl;
	u16	duration_id;
	u8	addr1[ETH_ALEN];
	u8	addr2[ETH_ALEN];
	u8	addr3[ETH_ALEN];
	u16	seq_ctl;
	u64	timestamp;
	u16	beacon_int;
	u16	capability;
	/*
	 * Additional info about essid, rates, atim and others is located here.
	 */
} __attribute__ ((packed));

struct _beacon_ie {
	u8	frame;
	u8	size;
	char	data; /* the data starts here but is size big */
};

/* Management Frame Information Element Types */
#define FRAME_IE_SSID			0x00
#define FRAME_IE_RATES			0x01
#define FRAME_IE_FH			0x02
#define FRAME_IE_DS			0x03
#define FRAME_IE_CF			0x04
#define FRAME_IE_TIM			0x05
#define FRAME_IE_IBSS			0x06

#define SET_IE_SSID			0x0001
#define SET_IE_RATES			0x0002
#define SET_IE_FH			0x0004
#define SET_IE_DS			0x0008
#define SET_IE_CF			0x0010
#define SET_IE_TIM			0x0020
#define SET_IE_IBSS			0x0040

/*
 * 802.11B capability features
 */
#define CAPABLE_ESS			0x0001
#define CAPABLE_IBSS			0x0002
#define CAPABLE_CF_POLLABLE		0x0004
#define CAPABLE_CF_POLL_REQ		0x0008
#define CAPABLE_PRIVACY			0x0010
/*
 * 802.11G capability features
 */
#define CAPABLE_SHORT_PREAMBLE		0x0020
#define CAPABLE_PBCC			0x0040
#define CAPABLE_AGILITY			0x0080
#define CAPABLE_PBCC_EXT_RATE		0x0100
#define CAPABLE_QOS			0x0200	/* defined in 802.11e d4.3 */
#define CAPABLE_SHORT_SLOT_TIME		0x0400
#define CAPABLE_APSD			0x0800	/* defined in 802.11e d4.3 */
#define CAPABLE_Q_ACK			0x1000	/* defined in 802.11e d4.3 */
#define CAPABLE_DSSS_OFDM		0x2000
#define CAPABLE_BLOCK_ACK		0x4000	/* defined in 802.11e d4.3 */

#endif /* __RT2X00_MGMT_H */
