/*
	rt2x00 SourceForge Project - <http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	Licensed under the GNU GPL
	Original code supplied under license from RaLink Inc, 2003
 */

/*
	Module Name: rt2x00_register.h

	Abstract: Register structures for RT2460 and RT2560 chips.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		<releasedate>	Initial Release.
 */

#ifndef __RT2X00_REGISTER_H
#define __RT2X00_REGISTER_H

#include <asm/io.h>

/*
 * Control/Status Registers(CSR)
 */
#define CSR0			0x0000		/* ASIC revision number */
#define CSR1			0x0004		/* System control register */
#define CSR2			0x0008		/* System admin status register (invalid) */
#define CSR3			0x000c		/* STA MAC address register 0 */
#define CSR4			0x0010		/* STA MAC address register 1 */
#define CSR5			0x0014		/* BSSID register 0 */
#define CSR6			0x0018		/* BSSID register 1 */
#define CSR7			0x001c		/* Interrupt source register */
#define CSR8			0x0020		/* Interrupt mask register */
#define CSR9			0x0024		/* Maximum frame length register */
#define CSR10			0x0028		/* RT2500 - WEP control register */
#define CSR11			0x002c		/* Back-off control register */
#define CSR12			0x0030		/* Synchronization configuration register 0 */
#define CSR13			0x0034		/* Synchronization configuration register 1 */
#define CSR14			0x0038		/* Synchronization control register */
#define CSR15			0x003c		/* Synchronization status register */
#define CSR16			0x0040		/* TSF timer register 0 */
#define CSR17			0x0044		/* TSF timer register 1 */
#define CSR18			0x0048		/* IFS timer register 0 */
#define CSR19			0x004c		/* IFS timer register 1 */
#define CSR20			0x0050		/* WakeUp register */
#define CSR21			0x0054		/* EEPROM control register */
#define CSR22			0x0058		/* CFP Control Register */

/* Transmit related CSRs */
#define TXCSR0			0x0060		/* TX control register */
#define TXCSR1			0x0064		/* TX configuration register */
#define TXCSR2			0x0068		/* TX descriptor configuratioon register */
#define TXCSR3			0x006c		/* TX Ring Base address register */
#define TXCSR4			0x0070		/* TX Atim Ring Base address register */
#define TXCSR5			0x0074		/* TX Prio Ring Base address register */
#define TXCSR6			0x0078		/* Beacon base address */
#define TXCSR7			0x007c		/* AutoResponder Control Register */
#define TXCSR9_2500		0x0094		/* RT2500 - OFDM TX BBP registers */

/* Receive related CSRs */
#define RXCSR0			0x0080		/* RX control register */
#define RXCSR1			0x0084		/* RX descriptor configuration register */
#define RXCSR2			0x0088		/* RX Ring base address register */
#define RXCSR3			0x0090		/* BBP ID register 0 */
#define RXCSR4_2400		0x0094		/* RT2400 - BBP ID register 1 */
#define ARCSR0			0x0098		/* Auto responder PLCP config register 1*/
#define ARCSR1			0x009c		/* Auto Responder PLCP config register 2 */

/* PCI control CSRs */
#define PCICSR			0x008c		/* PCI control register */

/* Statistic Register */
#define CNT0			0x00a0		/* FCS error count */
#define TIMECSR2		0x00a8
#define CNT1			0x00ac		/* PLCP error count */
#define CNT2			0x00b0		/* long error count */
#define TIMECSR3		0x00b4
#define CNT3			0x00b8		/* CCA false alarm count */
#define CNT4			0x00bc		/* Rx FIFO overflow count */
#define CNT5			0x00c0		/* Tx FIFO underrun count */

/* Baseband Control Register */
#define PWRCSR0			0x00c4		/* Power mode configuration */
#define PSCSR0			0x00c8		/* Power state transition time */
#define PSCSR1			0x00cc		/* Power state transition time */
#define PSCSR2			0x00d0		/* Power state transition time */
#define PSCSR3			0x00d4		/* Power state transition time */
#define PWRCSR1			0x00d8		/* Manual power control / status */
#define TIMECSR			0x00dc		/* Timer control */
#define MACCSR0			0x00e0		/* MAC configuration */
#define MACCSR1			0x00e4		/* MAC configuration */
#define RALINKCSR		0x00e8		/* Ralink Auto-reset register */
#define BCNCSR			0x00ec		/* Beacon interval control register */

/* BBP / RF / IF Control Register */
#define BBPCSR			0x00f0		/* BBP serial control */
#define RFCSR			0x00f4		/* RF serial control */
#define LEDCSR			0x00f8		/* LED control register */

#define SECCSR3			0x00fc		/* RT2500 - AES control register */

/* ASIC pointer information */
#define RXPTR			0x0100		/* Current RX ring address */
#define TXPTR			0x0104		/* Current Tx ring address */
#define PRIPTR			0x0108		/* Current Priority ring address */
#define ATIMPTR			0x010c		/* Current ATIM ring address */

#define TXACKCSR0		0x0110		/* RT2500 - TX ACK timeout */
#define ACKCNT0			0x0114		/* RT2500 - TX ACK timeout count */
#define ACKCNT1			0x0118		/* RT2500 - RX ACK timeout count */

/* GPIO and others */
#define GPIOCSR			0x0120		/* GPIO */
#define BBPPCSR_2400		0x0124		/* RT2400 - BBP Pin control register */
#define FIFOCSR0		0x0128		/* RT2500 - TX FIFO pointer */
#define FIFOCSR1		0x012c		/* RT2500 - RX FIFO pointer */
#define BCNCSR1			0x0130		/* Tx BEACON offset time, unit: 1 usec */
#define MACCSR2			0x0134		/* TX_PE to RX_PE delay time, unit: 1 PCI clock cycle */
#define TESTCSR			0x0138		/* RT2500 - TEST mode selection register */

#define ARCSR2			0x013c		/* 1 Mbps ACK/CTS PLCP */
#define ARCSR3			0x0140		/* 2 Mbps ACK/CTS PLCP */
#define ARCSR4			0x0144		/* 5.5 Mbps ACK/CTS PLCP */
#define ARCSR5			0x0148		/* 11 Mbps ACK/CTS PLCP */
#define ARTCSR0			0x014c		/* RT2500 - ACK/CTS payload consumed time for 1/2/5.5/11 mbps */
#define ARTCSR1			0x0150		/* RT2500 - OFDM ACK/CTS payload consumed time for 6/9/12/18 mbps */
#define ARTCSR2			0x0154		/* RT2500 - OFDM ACK/CTS payload consumed time for 24/36/48/54 mbps */
#define SECCSR1			0x0158		/* RT2500 - WEP control register */
#define BBPCSR1			0x015c		/* RT2500 - BBP TX configuration */
#define DBANDCSR0		0x0160		/* RT2500 - Dual band configuration register 0 */
#define DBANDCSR1		0x0164		/* RT2500 - Dual band configuration register 1 */
#define BBPPCSR_2500		0x0168		/* RT2500 - BBP Pin control register */
#define DBGSEL0			0x016c		/* RT2500 - MAC special debug mode selection register 0 */
#define DBGSEL1			0x0170		/* RT2500 - MAC special debug mode selection register 1 */
#define BISTCSR			0x0174		/* RT2500 - BBP BIST register */

#define MCAST0			0x0178		/* RT2500 - multicast filter register 0 */
#define MCAST1			0x017c		/* RT2500 - multicast filter register 1 */

#define UARTCSR0		0x0180		/* RT2500 - UART1 TX register */
#define UARTCSR1		0x0184		/* RT2500 - UART1 RX register */
#define UARTCSR3		0x0188		/* RT2500 - UART1 frame control register */
#define UARTCSR4		0x018c		/* RT2500 - UART1 buffer control register */
#define UART2CSR0		0x0190		/* RT2500 - UART2 TX register */
#define UART2CSR1		0x0194		/* RT2500 - UART2 RX register */
#define UART2CSR3		0x0198		/* RT2500 - UART2 frame control register */
#define UART2CSR4		0x019c		/* RT2500 - UART2 buffer control register */

/* EEPROM type */
#define EEPROM_TYPE_93c46	1
#define EEPROM_TYPE_93c66	0
#define EEPROM_WIDTH_93c46	6
#define EEPROM_WIDTH_93c66	8
#define EEPROM_DATA_OUT		0x10
#define EEPROM_DATA_IN		0x08
#define EEPROM_CHIP_SELECT	0x04
#define EEPROM_DATA_CLOCK	0x02
#define EEPROM_WRITE_OPCODE	0x05
#define EEPROM_READ_OPCODE	0x06

/* Device chip */
#define RT2400			0x0001
#define RT2500			0x0002

/* RF chip types */
#define RT2420			0x0000
#define RT2421			0x0001

#define RT2522			0x0000
#define RT2523			0x0001
#define RT2524			0x0002
#define RT2525			0x0003
#define RT2525E			0x0004
#define RT5222			0x0010

/*
 * TX descriptor format, Tx ring, Atim ring & Priority Ring
 */
struct _txd {
	/* Word 1. */
	u32			owner:1;
	u32			valid:1;
	u32			tx_result:3;
	u32			retry_count:3;
	u32			more_frag:1;
	u32			ack:1;
	u32			timestamp:1;
	u32			rts_ofdm:1;		/* For RT2400 set RTS, for RT2500 set OFDM. */
	u32			cipher_owner:1;		/* RT2500 only. */
	u32			ifs:2;
	u32			retry_mode:1;
	u32			data_byte_count:12;	/* RT2500 only. */
	u32			reserved0:4;

	/* Word 2. */
	u32			buffer_address;

	union {
		struct {
			/* Word 3. */
			u32	buffer_length:16;
			u32	data_byte_count:16;

			/* Word 4. */
			u32	plcp_signal:16;
			u32	plcp_service:16;

			/* Word 5. */
			u32	plcp_length_low:16;
			u32	plcp_length_high:16;

			/* Word 6. */
			u32	bbcr4:16;
			u32	agc_regid:7;
			u32	agc_regid_valid:1;
			u32	xxx_regid:7;
			u32	xxx_regid_valid:1;
		} rt2400;

		struct {
			/* Word 3. */
			u32	iv_offset:6;
			u32	aifs:2;
			u32	cw_min:4;
			u32	cw_max:4;
			u32	reserved1:16;

			/* Word 4. */
			u32	plcp_signal:8;
			u32	plcp_service:8;
			u32	plcp_length_low:8;
			u32	plcp_length_high:8;

			/* Word 5. */
			u32	iv;

			/* Word 6. */
			u32	eiv;
		} rt2500;
	} chip;

	/* Word 7-10. are reserved, use TXD_SPACE to align the rt2500 struct to the correct loaction.*/
#define TXD_SPACE	( 4 * sizeof(u32) )

	/* Word11. */
	struct {
		u32		rts:1;
		u32		tx_rate:7;
		u32		reserved6:24;
	} rt2500;
};

/*
 * Rx descriptor format, Rx Ring
 */
struct _rxd {
	/* Word 1. */
	u32			owner:1;
	u32			unicast:1;
	u32			multicast:1;
	u32			broadcast:1;
	u32			my_bss:1;
	u32			crc:1;
	u32			ofdm:1;		/* RT2500 only. */
	u32			error:1;
	u32			cipher_owner:1;
	u32			reserved0:7;
	u32			data_byte_count:12;
	u32			reserved:4;

	/* Word 2. */
	u32			buffer_address;

	/* Word 3. */
	union {
		struct {
			u32	buffer_length:16;
			u32	bbr0:8;
			u32	rssi:8;
		} rt2400;
	
		struct {
			u32	bbr0:8;
			u32	rssi:8;
			u32	reserved1:16;
		} rt2500;
	
	} chip;
};

/*
 * CSR Registers
 */

/*
 * CSR1: System control register
 */
union _csr1{
	struct {
		u32	softreset:1;		/* software reset bit, 1: reset, 0: normal */
		u32	hardreset:1;		/* hardware reset BBP, 1: reset, 0: release reset */
		u32	hostready:1;		/* host is ready after initialization, 1: ready */
		u32	reserved:29;
	} field;
	u32 word;
};

/*
 * CSR3: STA MAC register 0
 */
union _csr3{
	struct {
		u32	byte0:8;		/* mac address byte 0 */
		u32	byte1:8;		/* mac address byte 1 */
		u32	byte2:8;		/* mac address byte 2 */
		u32	byte3:8;		/* mac address byte 3 */
	} field;
	u32 word;
};

/*
 * CSR4: STA MAC register 1
 */
union _csr4{
	struct {
		u32	byte4:8;		/* mac address byte 4 */
		u32	byte5:8;		/* mac address byte 5 */
		u32	reserved:16;
	} field;
	u32 word;
};
 
/*
 * CSR5: BSSID register 0
 */
union _csr5{
	struct {
		u32	byte0:8;		/* bssid byte 0 */
		u32	byte1:8;		/* bssid byte 1 */
		u32	byte2:8;		/* bssid byte 2 */
		u32	byte3:8;		/* bssid byte 3 */
	} field;
	u32 word;
};

/*
 * CSR6: BSSID register 1
 */
union _csr6{
	struct {
		u32	byte4:8;		/* bssid byte 4 */
		u32	byte5:8;		/* bssid byte 5 */
		u32	reserved:16;
	} field;
	u32 word;
};

/*
 * CSR7: Interrupt source register
 * Write one to clear corresponding bit
 */
union _csr7{
	struct {
		u32	tbcn_expire:1;		/* beacon timer expired interrupt */
		u32	twake_expire:1;		/* wakeup timer expired interrupt */
		u32	tatimw_expire:1;	/* timer of atim window expired interrupt */
		u32	txdone_txring:1;	/* tx ring transmit done interrupt */
		u32	txdone_atimring:1;	/* atim ring transmit done interrupt */
		u32	txdone_prioring:1;	/* priority ring transmit done interrupt */
		u32	rxdone:1;		/* receive done interrupt */
		u32	decryption_done:1;	/* RT2500 - Decryption done interrupt */
		u32	encryption_done:1;	/* RT2500 - Encryption done interrupt */
		u32	uart1_tx_treshold:1;	/* RT2500 - UART1 TX reaches threshold */
		u32	uart1_rx_treshold:1;	/* RT2500 - UART1 RX reaches threshold */
		u32	uart1_idle_treshold:1;	/* RT2500 - UART1 IDLE over threshold */
		u32	uart1_tx_buff_error:1;	/* RT2500 - UART1 TX buffer error */
		u32	uart1_rx_buff_error:1;	/* RT2500 - UART1 RX buffer error */
		u32	uart2_tx_treshold:1;	/* RT2500 - UART2 TX reaches threshold */
		u32	uart2_rx_treshold:1;	/* RT2500 - UART2 RX reaches threshold */
		u32	uart2_idle_treshold:1;	/* RT2500 - UART2 IDLE over threshold */
		u32	uart2_tx_buff_error:1;	/* RT2500 - UART2 TX buffer error */
		u32	uart2_rx_buff_error:1;	/* RT2500 - UART2 RX buffer error */
		u32	timer_csr3_expire:1;	/* RT2500 - TIMECSR3 hardware timer expired (for 802.1H quiet period) */
		u32	reserved:12;
	} field;
	u32 word;
};

/*
 * CSR8: Interrupt Mask register
 * Write one to mask off interrupt
 */
union _csr8{
	struct {
		u32	tbcn_expire:1;		/* beacon timer expired interrupt */
		u32	twake_expire:1;		/* wakeup timer expired interrupt */
		u32	tatimw_expire:1;	/* timer of atim window expired interrupt */
		u32	txdone_txring:1;	/* tx ring transmit done interrupt */
		u32	txdone_atimring:1;	/* atim ring transmit done interrupt */
		u32	txdone_prioring:1;	/* priority ring transmit done interrupt */
		u32	rxdone:1;		/* receive done interrupt */
		u32	decryption_done:1;	/* RT2500 - Decryption done interrupt */
		u32	encryption_done:1;	/* RT2500 - Encryption done interrupt */
		u32	uart1_tx_treshold:1;	/* RT2500 - UART1 TX reaches threshold */
		u32	uart1_rx_treshold:1;	/* RT2500 - UART1 RX reaches threshold */
		u32	uart1_idle_treshold:1;	/* RT2500 - UART1 IDLE over threshold */
		u32	uart1_tx_buff_error:1;	/* RT2500 - UART1 TX buffer error */
		u32	uart1_rx_buff_error:1;	/* RT2500 - UART1 RX buffer error */
		u32	uart2_tx_treshold:1;	/* RT2500 - UART2 TX reaches threshold */
		u32	uart2_rx_treshold:1;	/* RT2500 - UART2 RX reaches threshold */
		u32	uart2_idle_treshold:1;	/* RT2500 - UART2 IDLE over threshold */
		u32	uart2_tx_buff_error:1;	/* RT2500 - UART2 TX buffer error */
		u32	uart2_rx_buff_error:1;	/* RT2500 - UART2 RX buffer error */
		u32	timer_csr3_expire:1;	/* RT2500 - TIMECSR3 hardware timer expired (for 802.1H quiet period) */
		u32	reserved:12;
	} field;
	u32 word;
};

/*
 * CSR9: Maximum frame length register
 */
union _csr9{
	struct {
		u32	reserved0:7;
		u32	max_frame_unit:5;	/* maximum frame length in 128b unit, default is 12 = 0xc. */
		u32	reserved1:20;
	} field;
	u32 word;
};

/*
 * CSR10: RT2500 - WEP control register
 */
union _csr10{
	struct {
		u32	kick_decrypt:1;		/* RT2500 - Kick decryption engine, self-clear */
		u32	one_shot_mode:1;	/* RT2500 - 1: One shot only mode, 0: ring mode */
		u32	desc_address:30;	/* RT2500 - Descriptor physical address of frame in one-shot-mode. */
	} field;
	u32 word;
};

/*
 * CSR11: Back-Off control register
 */
union _csr11{
	struct {
		u32	cwmin:4;		/* bit for cwmin. default cwmin is 31 (2^5 - 1). */
		u32	cwmax:4;		/* bit for cwmax, default cwmax is 1023 (2^10 - 1). */
		u32	slot_time:5;		/* slot time, default is 20us for 802.11b */
		u32	cw_select:1;		/* RT2500 - 1: CWmin/Cwmax select from register, 0:select from TxD */
		u32	reserved:2;
		u32	long_retry:8;		/* long retry count */
		u32	short_retry:8;		/* short retry count */
	} field;
	u32 word;
}; 

/*
 * CSR12: Synchronization configuration register 0
 * All units in 1/16 TU
 */
union _csr12{
	struct{
		u32	beacon_interval:16;	/* beacon interval, default is 100 tu */
		u32	cfpmax_duration:16;	/* cfp maximum duration, default is 100 tu */
	} field;
	u32 word;
};

/*
 * CSR13: Synchronization configuration register 1
 * All units in 1/16 TU
 */
union _csr13{
	struct {
		u32	atimw_duration:16;	/* atim window duration. */
		u32	cfp_period:8;		/* cfp period, default is 0 tu */
		u32	reserved:8;
	} field;
	u32 word;
};

/*
 * CSR14: Synchronization control register
 */
union _csr14{
	struct {
		u32	tsf_count:1;		/* enable tsf auto counting */
		u32	tsf_sync:2;		/* enable tsf sync, 00: disable, 01: infra, 10: ad-hoc mode */
		u32	tbcn:1;			/* enable tbcn with reload value */
		u32	tcfp:1;			/* enable tcfp & cfp / cp switching */
		u32	tatimw:1;		/* enable tatimw & atim window switching */
		u32	beacon_gen:1;		/* enable beacon generator */
		u32	reserved:1;
		u32	cfp_cnt_preload:8;	/* cfp count preload value */
		u32	tbcn_preload:16;	/* tbcn preload value in units of 64us*/
	} field;
	u32 word;
};

/*
 * CSR15: Synchronization status register
 */
union _csr15{
	struct {
		u32	cfp:1;			/* cfp period 1: ASIC currently in contention-free period */
		u32	atimw:1;		/* atim window period 1: ASIC currently in ATIM window */
		u32	beacon_sent:1;		/* beacon sent 1: beacon is send 0: beacon is not send */
		u32	reserved:29;
	}field;
	u32 word;
};

/*
 * Alias timer registers to self explaining name.
 */
#define LOW_TSFTIMER	(CSR16)			/* TSF timer register 0 */
#define HIGH_TSFTIMER	(CSR17)			/* TSF timer register 1 */

/*
 * CSR18: IFS Timer register 0
 */
union _csr18{
	struct {
		u32	sifs:9;			/* sifs, default is 10 us */
		u32	reserved0:7;
		u32	pifs:9;			/* pifs, default is 30 us */
		u32	reserved1:7;
	} field;
	u32 word;
};

/*
 * CSR19: IFS Timer register 1
 */
union _csr19{
	struct {
		u32	difs:16;		/* difs, default is 50 tu */
		u32	eifs:16;		/* eifs, default is 364 tu */
	} field;
	u32 word;
};

/*
 * CSR20: Wakeup timer register
 */
union _csr20{
	struct{
		u32	delay_after_tbcn:16;	/* delay after tbcn expired in units of 1/16 tu */
		u32	tbcn_before_wakeup:8;	/* number of beacon before wakeup */
		u32	autowake:1;		/* enable auto wakeup / sleep mechanism */
		u32	reserved:7;
	} field;
	u32 word;
};

/*
 * CSR21: EEPROM control register
 */
union _csr21{
	struct {
		u32	reload:1;		/* reload eeprom content, write one to reload, self-cleared. */
		u32	eepromsk:1;
		u32	eepromcs:1;
		u32	eepromdi:1;
		u32	eepromdo:1;
		u32	type:1;			/* 1: 93c46, 0:93c66 */
		u32	reserved:26;
	} field;
	u32 word;
};

/*
 * CSR22: CFP control register
 */
union _csr22{
	struct {
		u32	cfp_dur_remain:16;	/* cfp duration remain, in units of tu. */
		u32	reload_cfp_duration:1;	/* reload cfp duration remain, write one to reload/ */
		u32	reserved:15;
	} field;
	u32 word;
};

/*
 * TX / RX Registers
 */

/*
 * TXCSR0: TX Control Register 
 */
union _txcsr0 {
	struct {
		u32	kicktx:1;		/* kick tx ring */
		u32	kickatim:1;		/* kick atim ring */
		u32	kickprio:1;		/* kick priority ring */
		u32	abort:1;		/* abort all transmit related ring operation */
		u32	reserved:28;
	} field;
	u32 word;
};

/*
 * TXCSR1: TX Configuration Register
 */
union _txcsr1{
	struct {
		u32	ack_timeout:9;		/* ack timeout, default = sifs + 2*slottime + acktime @ 1mbps */
		u32	ack_consume_time:9;	/* ack consume time, default = sifs + acktime @ 1mbps */
		u32	tsf_offset:6;		/* insert tsf offset */
		u32	autoresponder:1;	/* enable auto responder which include ack & cts */
		u32	reserved:7;
	} field;
	u32 word;
};

/*
 * TXCSR2: Tx descriptor configuration register
 */
union _txcsr2{
	struct {
		u32	txd_size:8;		/* tx descriptor size, default is 32 */
		u32	num_txd:8;		/* number of txd in ring */
		u32	num_atim:8;		/* number of atim in ring */
		u32	num_prio:8;		/* number of priority in ring */
	} field;
	u32 word;
};

/*
 * Alias to all ring base registers. Easier to understand constant definition
 * within codes.
 */
#define TX_RING_REGISTER	(TXCSR3)	/* TX Ring Base address register */
#define ATIM_RING_REGISTER	(TXCSR4)	/* TX Atim Ring Base address register */
#define PRIO_RING_REGISTER	(TXCSR5)	/* TX Prio Ring Base address register */
#define BEACON_REGISTER		(TXCSR6)	/* Beacon base address */

/*
 * TXCSR7: Auto responder control register
 */
union _txcsr7{
	struct {
		u32	ar_powermanagment:1;	/* auto responder power management bit */
		u32	reserved:31;
	} field;
	u32 word;
};

/* 
 * TXCSR9: RT2500 - OFDM TX BBP registers
 * WARNING: May only be used by RT2500 adapters.
 */
union _txcsr9{
	struct {
		u32	ofdm_rate:8;		/* RT2500 - BBP rate field address for OFDM */
		u32	ofdm_service:8;		/* RT2500 - BBP service field address for OFDM */
		u32	ofdm_length_low:8;	/* RT2500 - BBP length low byte address for OFDM */
		u32	ofdm_length_high:8;	/* RT2500 - BBP length high byte address for OFDM */
	} field;
	u32 word;
};

/*
 * RXCSR0: RX Control Register
 */
union _rxcsr0{
	struct {
		u32	disable_rx:1;		/* disable rx engine */
		u32	drop_crc:1;		/* drop crc error */
		u32	drop_physical:1;	/* drop physical error */
		u32	drop_control:1;		/* drop control frame */
		u32	drop_not_to_me:1;	/* drop not to me unicast frame */
		u32	drop_tods:1;		/* drop frame tods bit is true */
		u32	drop_versionerr:1;	/* drop version error frame */
		u32	pass_crc:1;		/* pass all receive packet to host with crc attached */
		u32	pass_plcp:1;		/* RT2500 - Pass all receive packet with 4 bytes PLCP attached */
		u32	drop_mcast:1;		/* RT2500 - Drop multicast frames */
		u32	drop_bcast:1;		/* RT2500 - Drop broadcast frames */
		u32	enable_qos:1;		/* RT2500 - Accept QOS data frame format and parse the QOS field */
		u32	reserved:20;
	} field;
	u32 word;
};

/*
 * RXCSR1: RX descriptor configuration register
 */
union _rxcsr1{
	struct {
		u32	rxd_size:8;		/* rx descriptor size, default is 32b. */
		u32	num_rxd:8;		/* number of rxd in ring. */
		u32	reserved:16;
	} field;
	u32 word;
};

#define RX_RING_REGISTER	(RXCSR2)	/* RX Ring base address register */

/*
 * RXCSR3: BBP ID register for Rx operation
 */
union _rxcsr3{
	struct {
		u32	bbp_id0:7;		/* bbp register 0 id */
		u32	valid_bbp0:1;		/* bbp register 0 id is valid or not */
		u32	bbp_id1:7;		/* bbp register 1 id */
		u32	valid_bbp1:1;		/* bbp register 1 id is valid or not */
		u32	bbp_id2:7;		/* bbp register 2 id */
		u32	valid_bbp2:1;		/* bbp register 2 id is valid or not */
		u32	bbp_id3:7;		/* bbp register 3 id */
		u32	valid_bbp3:1;		/* bbp register 3 id is valid or not */
	} field;
	u32 word;
};

/*
 * RXCSR4: RT2400 - BBP ID register for Rx operation
 * WARNING: May only be accessed by RT2400 adapters.
 */
union _rxcsr4{
	struct {
		u32	bbp_id4:7;		/* RT2400 - bbp register 4 id */
		u32	valid_bbp4:1;		/* RT2400 - bbp register 4 id is valid or not */
		u32	bbp_id5:7;		/* RT2400 - bbp register 5 id */
		u32	valid_bbp5:1;		/* RT2400 - bbp register 5 id is valid or not */
		u32	reserved:16;
	} field;
	u32 word;
};

/*
 * ARCSR0: Auto Responder PLCP config register
 */
union _arcsr0{
	struct {
		u32	signal:8;		/* ack/cts bbp signal address */
		u32	service:8;		/* ack/cts bbp service address */
		u32	low_byte:8;		/* ack/cts bbp length low byte addres */
		u32	high_byte:8;		/* ack/cts bbp length high byte addres */
	} field;
	u32 word;
};

/*
 * ARCSR1: Auto Responder PLCP value basic rate bit mask
 */
union _arcsr1{
	struct {
		u32	basic_rate_mask:4;	/* basic rate bit mask bit0: 1M bit1: 2M bit2: 5.5M bit3: 11M*/
		u32	reserved0:4;
		u32	rxpe_reset_timer:8;	/* RX_PE auto_reset timer */
		u32	reserved1:16;
	} field;
	u32 word;
};

/*
 * PCISR: PCI control register
 */
union _pcicsr{
	struct {
		u32	bigendian:1;		/* 1: big endian, 0: little endian */
		u32	rx_threshold:2;		/* rx threshold in dw to start pci access */
						/* 01: 8dw, 10: 4dw, 11: 32dw, default 00: 16dw */
		u32	tx_threshold:2;		/* tx threshold in dw to start pci access */
						/* 01: 1dw, 10: 4dw, 11: store & forward, default 00: 0dw */
		u32	burst_length:2;		/* pci burst length */
						/* 01: 8dw, 10: 16dw, 11:32dw, default 00: 4dw */
		u32	enable_clk:1;		/* enable clk_run, */
						/* pci clock can't going down to non-operational */
		u32	read_multiple:1;	/* RT2500 - Enable memory read multiple */
		u32	write_invalid:1;	/* RT2500 - Enable memory write & invalid */
		u32	reserved:22;
	} field;
	u32 word;
};

/*
 * Miscellaneous Registers
 */
 
/*
 * PWRCSR1: Manual power control / status register
 * state: 00 deep_sleep, 01: sleep, 10: standby, 11: awake
 */
union _pwrcsr1{
	struct {
		u32	set_state:1;		/* set state. Write 1 to trigger, self cleared. */
		u32	bbp_desire_state:2;	/* BBP desired state */
		u32	rf_desire_state:2;	/* RF desired state */
		u32	bbp_curr_state:2;	/* BBP current state */
		u32	rf_curr_state:2;	/* RF current state */
		u32	put_to_sleep:1;		/* put to sleep. Write 1 to trigger, self cleared. */
		u32	reserved:22;
	} field;
	u32 word;
};

/*
 * TIMECSR: Timer control register
 */
union _timecsr{
	struct {
		u32	us_count:8;		/* 1 us timer count in units of clock cycles */
		u32	us_64_count:8;		/* 64 us timer count in units of 1 us timer */
		u32	beacon_expect:3;	/* Beacon expect window */
		u32	reserved:13;
	} field;
	u32 word;
};

/*
 * MACCSR1: MAC configuration register 1
 */
union _maccsr1{
	struct {
		u32	kick_rx:1;		/* kick one-shot rx in one-shot rx mode */
		u32	oneshot_rxmode:1;	/* enable one-shot rx mode for debugging */
		u32	bbprx_resetmode:1;	/* ralink bbp rx reset mode */
		u32	auto_txbbp:1;		/* auto tx logic access bbp control register */
		u32	auto_rxbbp:1;		/* auto rx logic access bbp control register */
		u32	loopback:2;		/* loopback mode. */
						/* 00: normal, 01: internal, 10: external, 11:rsvd. */
		u32	intersil_if:1;		/* intersil if calibration pin */
		u32	reserved:24;
	} field;
	u32 word;
};

/*
 * RALINKCSR: Ralink Rx auto-reset BBCR
 */
union _ralinkcsr{
	struct {
		u32	ar_bbp_data0:8;		/* auto reset bbp register 0 data */
		u32	ar_bbp_id0:7;		/* auto reset bbp register 0 id */
		u32	ar_bbpid_valid0:1;	/* auto reset bbp register 0 valid*/
		u32	ar_bbp_data1:8;		/* auto reset bbp register 1 data */
		u32	ar_bbp_id1:7;		/* auto reset bbp register 1 id */
		u32	ar_bbpid_valid1:1;	/* auto reset bbp register 1 valid*/
	} field;
	u32 word;
};

/*
 * BCNCSR: Beacon interval control register
 */
union _bcncsr{
	struct {
		u32	change:1;		/* write one to change beacon interval */
		u32	deltatime:4;		/* the delta time value */
		u32	num_beacon:8;		/* delta time value or number of beacon according to mode */
		u32	mode:2;			/* please refer to asic specs. */
		u32	plus:1;			/* plus or minus delta time value */
		u32	reserved:16;
	} field;
	u32 word;
};

/*
 * BBPCSR: BBP serial control register
 */
union _bbpcsr{
	struct {
		u32	value:8;		/* register value to program into bbp */
		u32	regnum:7;		/* selected bbp register */
		u32	busy:1;			/* 1: asic is busy execute bbp programming. */
		u32	writecontrol:1;		/* 1: write bbp, 0: read bbp */
		u32	reserved:15;
	} field;
	u32 word;
};

/*
 * RFCSR: RF serial control register
 */
union _rfcsr{
	struct {
		u32	rfregvalue:24;		/* register value (include register id)  */
						/* serial out to rf/if chip. */
		u32	numberofbits:5;		/* number of bits used in rfregvalue (i:20, rfmd:22) */
		u32	ifselect:1;		/* 1: select if to program, 0: select rf to program */
		u32	pll_ld:1;		/* rf pll_ld status */
		u32	busy:1;			/* 1: asic is busy execute rf programming. */
	} field;
	u32 word;
};

/*
 * LEDCSR: LED control register
 */
union _ledcsr{
	struct {
		u32	on_period:8;		/* on period, default 70ms */
		u32	off_period:8;		/* off period, default 30ms */
		u32	link:1;			/* 1: linkup, 0: linkoff */
		u32	activity:1;		/* 1: active, 0: idle */
		u32	link_polarity:1;	/* RT2500 - 0: active low, 1: active high */
		u32	activity_polarity:1;	/* RT2500 - 0: active low, 1: active high */
		u32	link_default:1;		/* RT2500 - LED A default value for "enable" state. 0: LED ON, 1: LED OFF */
		u32	reserved:13;
	}	field;
	u32 word;
};

/*
 * GPIOCSR: GPIO control register
 */
union _gpiocsr{
	struct {
		u32	bit0:1;
		u32	bit1:1;
		u32	bit2:1;
		u32	bit3:1;
		u32	bit4:1;
		u32	bit5:1;
		u32	bit6:1;
		u32	bit7:1;
		u32	dir0:1;			/* RT2500 */
		u32	dir1:1;			/* RT2500 */
		u32	dir2:1;			/* RT2500 */
		u32	dir3:1;			/* RT2500 */
		u32	dir4:1;			/* RT2500 */
		u32	dir5:1;			/* RT2500 */
		u32	dir6:1;			/* RT2500 */
		u32	dir7:1;			/* RT2500 */
		u32	reserved:16;
	} field;
	u32 word;
};

/*
 * BBPPCSR(_2400, _2500): BBP Pin control register
 */
#define BBPPCSR(__adapter)	( ((__adapter)->chip_id == RT2400) ? BBPPCSR_2400 : BBPPCSR_2500 )
union _bbppcsr{
	struct {
		u32	bbp_iddq:1;		/* BBP iddq pin control */
		u32	clock_disable:1;	/* clock disbale pin control */
		u32	clock_buff:1;		/* clock buffer enable */
		u32	reserved:29;
	} field;
	u32 word;
};
 
/*
 * BCNCSR1: Tx BEACON offset time control register
 */
union _bcncsr1{
	struct{
		u32	preload:16;		/* beacon timer offset in units of usec */
		u32	beacon_cwmin:4;		/* RT2500 - 2^CwMin */
		u32  	reserved:12;
	} field;
	u32 word;
};

/*
 * MACCSR2: TX_PE to RX_PE turn-around time control register
 */
union _maccsr2{
	struct {
		u32	delay:8;		/* RX_PE low width, in units of pci clock cycle */
		u32	reserved:24;
	} field;
	u32 word;
};

/*
 * ARCSR2: 1 Mbps ACK/CTS PLCP 
 */
union _arcsr2{
	struct {
		u32	signal:8;		/* Signal value */
		u32	service:8;		/* Service value; */
		u32	low_length:8;		/* Length low byte value */
		u32	high_length:8;		/* Length high byte value */
	} field;
	u32 word;
};

/*
 * ARCSR3: 2 Mbps ACK/CTS PLCP 
 */
union _arcsr3{
	struct {
		u32	signal:8;		/* Signal value */
		u32	service:8;		/* Service value; */
		u32	low_length:8;		/* Length low byte value */
		u32	high_length:8;		/* Length high byte value */
	} field;
	u32 word;
};

/*
 * ARCSR4: 5.5 Mbps ACK/CTS PLCP 
 */
union _arcsr4{
	struct {
		u32	signal:8;		/* Signal value */
		u32	service:8;		/* Service value; */
		u32	low_length:8;		/* Length low byte value */
		u32	high_lenght:8;		/* Length high byte value */
	} field;
	u32 word;
};

/*
 * ARCSR5: 11 Mbps ACK/CTS PLCP 
 */
union _arcsr5{
	struct {
		u32	signal:8;		/* Signal value */
		u32	service:8;		/* Service value; */
		u32	low_length:8;		/* Length low byte value */
		u32	high_length:8;		/* Length high byte value */
	} field;
	u32 word;
};

#define SECCSR0			( CSR10 )	/* RT2500 - WEP control register */

/*
 * SECCSR1: RT2500 - WEP control register 
 */
union _seccsr1{
	struct {
		u32	kick_encypt:1;		/* RT2500 - Kick encryption engine, self-clear */
		u32	one_shot_mode:1;	/* RT2500 - 1: One shot only mode, 0: ring mode */
		u32	desc_address:30;	/* RT2500 - Descriptor physical address of frame in one-shot-mode. */
	} field;
	u32 word;
};

struct _eeprom_antenna {
	u16	num_of_antenna:2;	/* Number of antenna. */
	u16	tx_default_antenna:2;	/* Default antenna - 0: diversity, 1: Antenna-A, 2: Antenna-B. */
	u16	rx_default_antenna:2;	/* Default antenna - 0: diversity, 1: Antenna-A, 2: Antenna-B. */
	u16	rt2400_rf_type:1;	/* RT2400 - rf_type of this adapter. */
	u16	reserved0:3;
	u16	hardware_radio:1;	/* Hardware controlled radio enabled. */
	u16	rt2500_rf_type:5;	/* RT2500 - rf_type of this adapter. */
};

struct _eeprom_bbp {
	u16	reg_id:8;
	u16	value:8;
};

struct _eeprom_txpower {
	u16	tx_power1:8;
	u16	tx_power2:8;
};

/*
 * Register Handlers.
 */
/*
 * rt2x00_register_read & rt2x00_register_read
 * Functions for PCI register read / write
 *
 * These 2 functions must be called with adapter->lock held!
 *
 * The registers are 32 bit little endian values.
 * On register read and write, values should be converted to and from cpu byte ordering.
 */
static inline void rt2x00_register_read(struct _rt2x00_adapter *adapter, unsigned long offset, u32 *value)
{
	*value = le32_to_cpu(readl(adapter->register_base_address + offset));
}

static inline void rt2x00_register_write(struct _rt2x00_adapter *adapter, unsigned long offset, u32 value)
{
	writel(cpu_to_le32(value), adapter->register_base_address + offset);
}

/*
 * BBP & RF are using indirect access. Before write any value into it.
 * We have to make sure there is no outstanding command pending via checking busy bit.
 */
static inline void rt2x00_bbp_regwrite(struct _rt2x00_adapter *adapter, u8 reg_id, u8 value)
{
	union _bbpcsr	reg;
	u8		counter = 0;

	do {
		rt2x00_register_read(adapter, BBPCSR, &reg.word);
		if (!reg.field.busy)
			break;
		counter++;
	} while (counter < MAX_REGISTER_BUSY_COUNT);

	if (counter == MAX_REGISTER_BUSY_COUNT){
		ERROR("BBPCSR register busy.\n");
	}

	/* Write value. */
	reg.field.value = value;

	/* Set register index. */
	reg.field.regnum = reg_id;

	/* At register write set write_control (bit 17) and busy check (bit 16). */
	reg.field.busy = 1;
	reg.field.writecontrol = 1;

	rt2x00_register_write(adapter, BBPCSR, reg.word);
}

static inline void rt2x00_bbp_regread(struct _rt2x00_adapter *adapter, u8 reg_id, u8 *value)
{
	union _bbpcsr	reg;
	u32		counter = 0;

	/*
	 * To read the correct BBP register, we have to write the requested register to the register,
	 * we also have to set the busy bit.
	 */
	reg.word = 0;
	reg.field.writecontrol = 0;
	reg.field.regnum = reg_id;
	reg.field.busy = 1;

	rt2x00_register_write(adapter, BBPCSR, reg.word);

	do {
		rt2x00_register_read(adapter, BBPCSR, &reg.word);
		if (!reg.field.busy)
			break;
		counter++;
	} while (counter < MAX_REGISTER_BUSY_COUNT);

	if (counter == MAX_REGISTER_BUSY_COUNT){
		ERROR("BBPCSR register busy.\n");
		*value = 0xff;
	} else
		*value = reg.field.value;
}

static inline void rt2x00_rf_regwrite(struct _rt2x00_adapter *adapter, u32 value)
{
	union _rfcsr	reg;
	u8		counter = 0;

	do {
		rt2x00_register_read(adapter, RFCSR, &reg.word);
		if (!reg.field.busy)
			break;
		counter++;
	} while (counter < MAX_REGISTER_BUSY_COUNT);

	if (counter == MAX_REGISTER_BUSY_COUNT){
		ERROR("RFCSR register busy.\n");
	}

	/*
	 * Write value to the register. 
	 * This will automaticly set the number of bits and busy bit.
	 */
	rt2x00_register_write(adapter, RFCSR, value);
}

/*
 * EEPROM access funtion.
 */
u16 rt2x00_eeprom_read16(struct _rt2x00_adapter *adapter, u8 offset, u8 bus_width);

static inline void rt2x00_eeprom_regread(struct _rt2x00_adapter *adapter, u8 reg_offset, u8 words, u16 *value)
{
	union _csr21	csr21;
	u8		eeprom_width;
	u16		reg = 0x0000;

	/*
	 * First we have to identify the register type.
	 */
	rt2x00_register_read(adapter, CSR21, &csr21.word);

	if(csr21.field.type == EEPROM_TYPE_93c46)
		eeprom_width = EEPROM_WIDTH_93c46;
	else
		eeprom_width = EEPROM_WIDTH_93c66;

	/*
	 * Start reading EEPROM.
	 */
	do{
		reg = rt2x00_eeprom_read16(adapter, reg_offset, eeprom_width);

		*value = le16_to_cpu(reg);
		reg_offset += sizeof(u16);
		value++;
	} while(--words);
}

#endif /* __RT2X00_REGISTER_H */
